#!/usr/bin/python3
import json
import logging
import logging.config
import os

from PyQt5 import QtCore, QtDBus

DB_DIR = os.path.expanduser("~/.config/qoob/")
global name; name = "qoob"


def copyDict(dictionnary):
    # from https://writeonly.wordpress.com/2009/05/07/deepcopy-is-a-pig-for-simple-data
    out = dict().fromkeys(dictionnary)
    for key, value in dictionnary.items():
        try:
            out[key] = value.copy()  # dicts, sets
        except AttributeError:
            try:
                out[key] = value[:]  # lists, tuples, strings, unicode
            except TypeError:
                out[key] = value  # ints
    return out


class Database(object):
    def __init__(self, dbFile):
        self.dbFile = DB_DIR + dbFile + ".json"
        if not os.path.isdir(DB_DIR):
            os.mkdir(DB_DIR)
        if os.path.isfile(self.dbFile) and os.stat(self.dbFile).st_size > 0:
            self.load()
        else:
            self.db = {}
            with open(self.dbFile, "w") as f:
                f.write(json.dumps(self.db, indent=2, sort_keys=False))

    def load(self):
        with open(self.dbFile, "r") as f:
            self.db = json.load(f)

    def save(self):
        with open(self.dbFile, "w") as f:
            f.write(json.dumps(self.db, indent=2, sort_keys=False))


class Logger:
    def __init__(self, name, path):
        if not os.path.isdir(f"{path}"):
            os.mkdir(f"{path}")
        LOGGER_CONFIG = {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters':
            {
                'default':
                {
                    'format': "[%(asctime)s] [%(levelname)s] [%(name)s] %(message)s",
                    'datefmt': "%H:%M:%S",
                },
            },
            'handlers':
            {
                'file':
                {
                    'level': 'DEBUG',
                    'formatter': 'default',
                    'class': 'logging.FileHandler',
                    'filename': f'{path}{name}.log',
                    'mode': 'w',
                },
                'screen':
                {
                    'level': 'INFO',
                    'formatter': 'default',
                    'class': 'logging.StreamHandler',
                },
            },
            'loggers':
            {
                '':
                {
                    'handlers': ['file', 'screen'],
                    'level': 'DEBUG',
                    'propagate': True,
                },
            }
        }
        logging.config.dictConfig(LOGGER_CONFIG)
        #sys.excepthook = self._exceptHook

    def _exceptHook(self, type_, error, traceback):
        tb = traceback
        logging.critical(f"{type_} {error}".rstrip())
        while "tb_next" in dir(tb):
            logging.critical(f"{tb.tb_frame}")
            tb = tb.tb_next
        sys.__excepthook__(type_, error, traceback)  # Call default hook

    def new(self, name):
        return logging.getLogger(name)


class QDBusObject(QtCore.QObject):
    def __init__(self, parent):
        QtCore.QObject.__init__(self)
        self.__dbusAdaptor = QDBusServerAdapter(self, parent)
        self.start()

    def start(self):
        bus = QtDBus.QDBusConnection.sessionBus()
        bus.registerObject(f"/org/{name}/session", self)
        bus.registerService(f"org.{name}.session")
        return bus


class QDBusServerAdapter(QtDBus.QDBusAbstractAdaptor):
    QtCore.Q_CLASSINFO("D-Bus Interface", f"org.{name}.session")
    QtCore.Q_CLASSINFO("D-Bus Introspection",
    f'<interface name="org.{name}.session">\n'
    '  <method name="parse">\n'
    '    <arg direction="in" type="s" name="cmd"/>\n'
    '  </method>\n'
    '</interface>\n')

    def __init__(self, server, parent):
        super().__init__(server)
        self.parent = parent

    @QtCore.pyqtSlot(str)
    def parse(self, cmd):
        # Serialize the string of commands
        if cmd:
            commands = {}
            current = ""
            for arg in cmd.split("%"):
                if arg.startswith("-"):
                    current = arg.lstrip("-")
                elif current:
                    commands[current].append(arg)
                if current not in commands:
                    commands[current] = []
            self.parent.parseCommands(commands)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qoob/ui/preferences.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(600, 500)
        self.gridLayout_8 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.sideMenuList = QtWidgets.QListWidget(Dialog)
        self.sideMenuList.setMinimumSize(QtCore.QSize(150, 0))
        self.sideMenuList.setMaximumSize(QtCore.QSize(150, 16777215))
        self.sideMenuList.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.sideMenuList.setObjectName("sideMenuList")
        item = QtWidgets.QListWidgetItem()
        self.sideMenuList.addItem(item)
        item = QtWidgets.QListWidgetItem()
        self.sideMenuList.addItem(item)
        item = QtWidgets.QListWidgetItem()
        self.sideMenuList.addItem(item)
        item = QtWidgets.QListWidgetItem()
        self.sideMenuList.addItem(item)
        self.gridLayout.addWidget(self.sideMenuList, 0, 0, 1, 2)
        self.resetButton = QtWidgets.QPushButton(Dialog)
        self.resetButton.setObjectName("resetButton")
        self.gridLayout.addWidget(self.resetButton, 1, 0, 1, 2)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Apply|QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 1, 2, 1, 1)
        self.stackedWidget = QtWidgets.QStackedWidget(Dialog)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_3 = QtWidgets.QWidget()
        self.page_3.setObjectName("page_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.page_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBox_4 = QtWidgets.QGroupBox(self.page_3)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.dbAddButton = QtWidgets.QPushButton(self.groupBox_4)
        self.dbAddButton.setObjectName("dbAddButton")
        self.gridLayout_3.addWidget(self.dbAddButton, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 2, 2, 1, 1)
        self.dbDeleteButton = QtWidgets.QPushButton(self.groupBox_4)
        self.dbDeleteButton.setObjectName("dbDeleteButton")
        self.gridLayout_3.addWidget(self.dbDeleteButton, 2, 1, 1, 1)
        self.dbResetButton = QtWidgets.QToolButton(self.groupBox_4)
        self.dbResetButton.setMinimumSize(QtCore.QSize(30, 0))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("icons/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.dbResetButton.setIcon(icon)
        self.dbResetButton.setObjectName("dbResetButton")
        self.gridLayout_3.addWidget(self.dbResetButton, 2, 3, 1, 1)
        self.dbTree = QtWidgets.QTreeWidget(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dbTree.sizePolicy().hasHeightForWidth())
        self.dbTree.setSizePolicy(sizePolicy)
        self.dbTree.setAlternatingRowColors(True)
        self.dbTree.setRootIsDecorated(False)
        self.dbTree.setObjectName("dbTree")
        self.dbTree.header().setDefaultSectionSize(220)
        self.dbTree.header().setStretchLastSection(True)
        self.gridLayout_3.addWidget(self.dbTree, 1, 0, 1, 4)
        self.gridLayout_2.addWidget(self.groupBox_4, 2, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.page_3)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.deleteConfirmBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.deleteConfirmBox.setObjectName("deleteConfirmBox")
        self.gridLayout_7.addWidget(self.deleteConfirmBox, 1, 0, 1, 3)
        self.resumePlaybackBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.resumePlaybackBox.setObjectName("resumePlaybackBox")
        self.gridLayout_7.addWidget(self.resumePlaybackBox, 0, 0, 1, 3)
        self.cleanFolderBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.cleanFolderBox.setObjectName("cleanFolderBox")
        self.gridLayout_7.addWidget(self.cleanFolderBox, 2, 0, 1, 3)
        self.label_3 = QtWidgets.QLabel(self.groupBox_3)
        self.label_3.setObjectName("label_3")
        self.gridLayout_7.addWidget(self.label_3, 5, 0, 1, 1)
        self.fileManagerLine = QtWidgets.QLineEdit(self.groupBox_3)
        self.fileManagerLine.setObjectName("fileManagerLine")
        self.gridLayout_7.addWidget(self.fileManagerLine, 5, 1, 1, 2)
        self.gridLayout_2.addWidget(self.groupBox_3, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_2.addItem(spacerItem1, 1, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_3)
        self.page_4 = QtWidgets.QWidget()
        self.page_4.setObjectName("page_4")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.page_4)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.groupBox_2 = QtWidgets.QGroupBox(self.page_4)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.trayIconBox = QtWidgets.QCheckBox(self.groupBox_2)
        self.trayIconBox.setObjectName("trayIconBox")
        self.gridLayout_5.addWidget(self.trayIconBox, 0, 0, 1, 1)
        self.trayMinimizeBox = QtWidgets.QCheckBox(self.groupBox_2)
        self.trayMinimizeBox.setObjectName("trayMinimizeBox")
        self.gridLayout_5.addWidget(self.trayMinimizeBox, 0, 1, 1, 1)
        self.gridLayout_9.addWidget(self.groupBox_2, 2, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_9.addItem(spacerItem2, 3, 0, 1, 1)
        self.groupBox_10 = QtWidgets.QGroupBox(self.page_4)
        self.groupBox_10.setObjectName("groupBox_10")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.groupBox_10)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.lightThemeRadio = QtWidgets.QRadioButton(self.groupBox_10)
        self.lightThemeRadio.setObjectName("lightThemeRadio")
        self.gridLayout_12.addWidget(self.lightThemeRadio, 0, 0, 1, 1)
        self.darkThemeRadio = QtWidgets.QRadioButton(self.groupBox_10)
        self.darkThemeRadio.setObjectName("darkThemeRadio")
        self.gridLayout_12.addWidget(self.darkThemeRadio, 0, 1, 1, 1)
        self.gridLayout_9.addWidget(self.groupBox_10, 0, 0, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_9.addItem(spacerItem3, 1, 0, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_9.addItem(spacerItem4, 5, 0, 1, 1)
        self.groupBox_7 = QtWidgets.QGroupBox(self.page_4)
        self.groupBox_7.setObjectName("groupBox_7")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_7)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.artTriggerSelectionBox = QtWidgets.QCheckBox(self.groupBox_7)
        self.artTriggerSelectionBox.setObjectName("artTriggerSelectionBox")
        self.gridLayout_10.addWidget(self.artTriggerSelectionBox, 4, 0, 1, 1)
        self.line_2 = QtWidgets.QFrame(self.groupBox_7)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_10.addWidget(self.line_2, 2, 1, 3, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_10.addItem(spacerItem5, 1, 0, 1, 1)
        self.artSizeBox = QtWidgets.QSpinBox(self.groupBox_7)
        self.artSizeBox.setMaximum(270)
        self.artSizeBox.setSingleStep(10)
        self.artSizeBox.setObjectName("artSizeBox")
        self.gridLayout_10.addWidget(self.artSizeBox, 3, 2, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_10.addItem(spacerItem6, 3, 3, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.groupBox_7)
        self.label_5.setObjectName("label_5")
        self.gridLayout_10.addWidget(self.label_5, 2, 2, 1, 2)
        self.label = QtWidgets.QLabel(self.groupBox_7)
        self.label.setObjectName("label")
        self.gridLayout_10.addWidget(self.label, 2, 0, 1, 1)
        self.artTriggerMediaBox = QtWidgets.QCheckBox(self.groupBox_7)
        self.artTriggerMediaBox.setObjectName("artTriggerMediaBox")
        self.gridLayout_10.addWidget(self.artTriggerMediaBox, 3, 0, 1, 1)
        self.artShowEmptyBox = QtWidgets.QCheckBox(self.groupBox_7)
        self.artShowEmptyBox.setObjectName("artShowEmptyBox")
        self.gridLayout_10.addWidget(self.artShowEmptyBox, 0, 0, 1, 4)
        self.gridLayout_9.addWidget(self.groupBox_7, 4, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_4)
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.gridLayout_18 = QtWidgets.QGridLayout(self.page)
        self.gridLayout_18.setObjectName("gridLayout_18")
        self.groupBox_6 = QtWidgets.QGroupBox(self.page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_6.sizePolicy().hasHeightForWidth())
        self.groupBox_6.setSizePolicy(sizePolicy)
        self.groupBox_6.setMaximumSize(QtCore.QSize(16777215, 200))
        self.groupBox_6.setObjectName("groupBox_6")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_6)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.sortSelectedList = QtWidgets.QListWidget(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sortSelectedList.sizePolicy().hasHeightForWidth())
        self.sortSelectedList.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(7)
        self.sortSelectedList.setFont(font)
        self.sortSelectedList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.sortSelectedList.setObjectName("sortSelectedList")
        self.gridLayout_6.addWidget(self.sortSelectedList, 1, 0, 3, 1)
        spacerItem7 = QtWidgets.QSpacerItem(21, 89, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_6.addItem(spacerItem7, 3, 1, 1, 1)
        self.sortRemoveButton = QtWidgets.QToolButton(self.groupBox_6)
        self.sortRemoveButton.setArrowType(QtCore.Qt.RightArrow)
        self.sortRemoveButton.setObjectName("sortRemoveButton")
        self.gridLayout_6.addWidget(self.sortRemoveButton, 1, 1, 1, 1)
        self.sortAddButton = QtWidgets.QToolButton(self.groupBox_6)
        self.sortAddButton.setArrowType(QtCore.Qt.LeftArrow)
        self.sortAddButton.setObjectName("sortAddButton")
        self.gridLayout_6.addWidget(self.sortAddButton, 2, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.groupBox_6)
        self.label_10.setObjectName("label_10")
        self.gridLayout_6.addWidget(self.label_10, 0, 2, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox_6)
        self.label_4.setObjectName("label_4")
        self.gridLayout_6.addWidget(self.label_4, 0, 0, 1, 1)
        self.sortAvailableList = QtWidgets.QListWidget(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sortAvailableList.sizePolicy().hasHeightForWidth())
        self.sortAvailableList.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(7)
        self.sortAvailableList.setFont(font)
        self.sortAvailableList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.sortAvailableList.setObjectName("sortAvailableList")
        self.gridLayout_6.addWidget(self.sortAvailableList, 1, 2, 3, 1)
        self.sortByDefaultBox = QtWidgets.QCheckBox(self.groupBox_6)
        self.sortByDefaultBox.setObjectName("sortByDefaultBox")
        self.gridLayout_6.addWidget(self.sortByDefaultBox, 4, 0, 1, 1)
        self.gridLayout_18.addWidget(self.groupBox_6, 2, 0, 1, 1)
        self.groupBox_5 = QtWidgets.QGroupBox(self.page)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_14 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.stripTitlesBox = QtWidgets.QCheckBox(self.groupBox_5)
        self.stripTitlesBox.setObjectName("stripTitlesBox")
        self.gridLayout_14.addWidget(self.stripTitlesBox, 1, 0, 1, 2)
        self.label_12 = QtWidgets.QLabel(self.groupBox_5)
        self.label_12.setObjectName("label_12")
        self.gridLayout_14.addWidget(self.label_12, 3, 0, 1, 1)
        self.titleFormatLine = QtWidgets.QLineEdit(self.groupBox_5)
        self.titleFormatLine.setObjectName("titleFormatLine")
        self.gridLayout_14.addWidget(self.titleFormatLine, 3, 1, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.groupBox_5)
        self.label_11.setObjectName("label_11")
        self.gridLayout_14.addWidget(self.label_11, 4, 0, 1, 1)
        self.notificationFormatLine = QtWidgets.QLineEdit(self.groupBox_5)
        self.notificationFormatLine.setObjectName("notificationFormatLine")
        self.gridLayout_14.addWidget(self.notificationFormatLine, 4, 1, 1, 1)
        self.expandLibraryBox = QtWidgets.QCheckBox(self.groupBox_5)
        self.expandLibraryBox.setObjectName("expandLibraryBox")
        self.gridLayout_14.addWidget(self.expandLibraryBox, 0, 0, 1, 2)
        self.label_2 = QtWidgets.QLabel(self.groupBox_5)
        self.label_2.setObjectName("label_2")
        self.gridLayout_14.addWidget(self.label_2, 2, 0, 1, 1)
        self.tooltipFormatLine = QtWidgets.QLineEdit(self.groupBox_5)
        self.tooltipFormatLine.setObjectName("tooltipFormatLine")
        self.gridLayout_14.addWidget(self.tooltipFormatLine, 2, 1, 1, 1)
        self.gridLayout_18.addWidget(self.groupBox_5, 0, 0, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_18.addItem(spacerItem8, 3, 0, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_18.addItem(spacerItem9, 1, 0, 1, 1)
        self.stackedWidget.addWidget(self.page)
        self.page_5 = QtWidgets.QWidget()
        self.page_5.setObjectName("page_5")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.page_5)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.groupBox = QtWidgets.QGroupBox(self.page_5)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_13 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.label_20 = QtWidgets.QLabel(self.groupBox)
        self.label_20.setMinimumSize(QtCore.QSize(50, 0))
        self.label_20.setObjectName("label_20")
        self.gridLayout_13.addWidget(self.label_20, 0, 3, 1, 1)
        self.popupFontFamilyCombo = QtWidgets.QFontComboBox(self.groupBox)
        self.popupFontFamilyCombo.setMaximumSize(QtCore.QSize(160, 16777215))
        self.popupFontFamilyCombo.setObjectName("popupFontFamilyCombo")
        self.gridLayout_13.addWidget(self.popupFontFamilyCombo, 0, 4, 1, 1)
        spacerItem10 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_13.addItem(spacerItem10, 0, 2, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.groupBox)
        self.label_19.setMinimumSize(QtCore.QSize(40, 0))
        self.label_19.setObjectName("label_19")
        self.gridLayout_13.addWidget(self.label_19, 0, 0, 1, 1)
        self.popupFontSizeBox = QtWidgets.QSpinBox(self.groupBox)
        self.popupFontSizeBox.setObjectName("popupFontSizeBox")
        self.gridLayout_13.addWidget(self.popupFontSizeBox, 0, 1, 1, 1)
        self.gridLayout_11.addWidget(self.groupBox, 2, 0, 1, 2)
        self.groupBox_9 = QtWidgets.QGroupBox(self.page_5)
        self.groupBox_9.setObjectName("groupBox_9")
        self.gridLayout_15 = QtWidgets.QGridLayout(self.groupBox_9)
        self.gridLayout_15.setObjectName("gridLayout_15")
        self.popupVOffsetBox = QtWidgets.QSpinBox(self.groupBox_9)
        self.popupVOffsetBox.setMinimum(-9999)
        self.popupVOffsetBox.setMaximum(9999)
        self.popupVOffsetBox.setObjectName("popupVOffsetBox")
        self.gridLayout_15.addWidget(self.popupVOffsetBox, 1, 4, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox_9)
        self.label_7.setObjectName("label_7")
        self.gridLayout_15.addWidget(self.label_7, 1, 2, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.groupBox_9)
        self.label_24.setAlignment(QtCore.Qt.AlignCenter)
        self.label_24.setObjectName("label_24")
        self.gridLayout_15.addWidget(self.label_24, 3, 0, 1, 1)
        self.popupHeightBox = QtWidgets.QSpinBox(self.groupBox_9)
        self.popupHeightBox.setMaximum(400)
        self.popupHeightBox.setSingleStep(5)
        self.popupHeightBox.setObjectName("popupHeightBox")
        self.gridLayout_15.addWidget(self.popupHeightBox, 3, 4, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_15.addItem(spacerItem11, 0, 3, 1, 1)
        self.popupHOffsetBox = QtWidgets.QSpinBox(self.groupBox_9)
        self.popupHOffsetBox.setMinimum(-9999)
        self.popupHOffsetBox.setMaximum(9999)
        self.popupHOffsetBox.setObjectName("popupHOffsetBox")
        self.gridLayout_15.addWidget(self.popupHOffsetBox, 0, 4, 1, 1)
        self.popupWidthBox = QtWidgets.QSpinBox(self.groupBox_9)
        self.popupWidthBox.setMaximum(400)
        self.popupWidthBox.setSingleStep(5)
        self.popupWidthBox.setObjectName("popupWidthBox")
        self.gridLayout_15.addWidget(self.popupWidthBox, 2, 4, 1, 1)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_15.addItem(spacerItem12, 0, 1, 1, 1)
        self.label_22 = QtWidgets.QLabel(self.groupBox_9)
        self.label_22.setObjectName("label_22")
        self.gridLayout_15.addWidget(self.label_22, 2, 2, 1, 1)
        self.label_25 = QtWidgets.QLabel(self.groupBox_9)
        self.label_25.setObjectName("label_25")
        self.gridLayout_15.addWidget(self.label_25, 3, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.groupBox_9)
        self.label_6.setObjectName("label_6")
        self.gridLayout_15.addWidget(self.label_6, 0, 2, 1, 1)
        self.frame = QtWidgets.QFrame(self.groupBox_9)
        self.frame.setMaximumSize(QtCore.QSize(75, 75))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout_16 = QtWidgets.QGridLayout(self.frame)
        self.gridLayout_16.setObjectName("gridLayout_16")
        self.middleRightRadio = QtWidgets.QRadioButton(self.frame)
        self.middleRightRadio.setText("")
        self.middleRightRadio.setObjectName("middleRightRadio")
        self.gridLayout_16.addWidget(self.middleRightRadio, 1, 2, 1, 1)
        self.bottomRightRadio = QtWidgets.QRadioButton(self.frame)
        self.bottomRightRadio.setText("")
        self.bottomRightRadio.setObjectName("bottomRightRadio")
        self.gridLayout_16.addWidget(self.bottomRightRadio, 2, 2, 1, 1)
        self.middleCenterRadio = QtWidgets.QRadioButton(self.frame)
        self.middleCenterRadio.setText("")
        self.middleCenterRadio.setObjectName("middleCenterRadio")
        self.gridLayout_16.addWidget(self.middleCenterRadio, 1, 1, 1, 1)
        self.topLeftRadio = QtWidgets.QRadioButton(self.frame)
        self.topLeftRadio.setText("")
        self.topLeftRadio.setObjectName("topLeftRadio")
        self.gridLayout_16.addWidget(self.topLeftRadio, 0, 0, 1, 1)
        self.bottomCenterRadio = QtWidgets.QRadioButton(self.frame)
        self.bottomCenterRadio.setText("")
        self.bottomCenterRadio.setObjectName("bottomCenterRadio")
        self.gridLayout_16.addWidget(self.bottomCenterRadio, 2, 1, 1, 1)
        self.bottomLeftRadio = QtWidgets.QRadioButton(self.frame)
        self.bottomLeftRadio.setText("")
        self.bottomLeftRadio.setObjectName("bottomLeftRadio")
        self.gridLayout_16.addWidget(self.bottomLeftRadio, 2, 0, 1, 1)
        self.middleLeftRadio = QtWidgets.QRadioButton(self.frame)
        self.middleLeftRadio.setText("")
        self.middleLeftRadio.setObjectName("middleLeftRadio")
        self.gridLayout_16.addWidget(self.middleLeftRadio, 1, 0, 1, 1)
        self.topRightRadio = QtWidgets.QRadioButton(self.frame)
        self.topRightRadio.setText("")
        self.topRightRadio.setObjectName("topRightRadio")
        self.gridLayout_16.addWidget(self.topRightRadio, 0, 2, 1, 1)
        self.topCenterRadio = QtWidgets.QRadioButton(self.frame)
        self.topCenterRadio.setText("")
        self.topCenterRadio.setObjectName("topCenterRadio")
        self.gridLayout_16.addWidget(self.topCenterRadio, 0, 1, 1, 1)
        self.gridLayout_15.addWidget(self.frame, 0, 0, 3, 1)
        self.gridLayout_11.addWidget(self.groupBox_9, 4, 0, 1, 2)
        spacerItem13 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_11.addItem(spacerItem13, 1, 0, 1, 2)
        self.groupBox_8 = QtWidgets.QGroupBox(self.page_5)
        self.groupBox_8.setObjectName("groupBox_8")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_8)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_21 = QtWidgets.QLabel(self.groupBox_8)
        self.label_21.setObjectName("label_21")
        self.gridLayout_4.addWidget(self.label_21, 2, 0, 1, 1)
        self.popupBackgroundButton = QtWidgets.QToolButton(self.groupBox_8)
        self.popupBackgroundButton.setObjectName("popupBackgroundButton")
        self.gridLayout_4.addWidget(self.popupBackgroundButton, 1, 4, 1, 1)
        self.popupForegroundButton = QtWidgets.QToolButton(self.groupBox_8)
        self.popupForegroundButton.setObjectName("popupForegroundButton")
        self.gridLayout_4.addWidget(self.popupForegroundButton, 2, 4, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.groupBox_8)
        self.label_13.setObjectName("label_13")
        self.gridLayout_4.addWidget(self.label_13, 2, 3, 1, 1)
        self.popupDurationBox = QtWidgets.QSpinBox(self.groupBox_8)
        self.popupDurationBox.setMaximum(10)
        self.popupDurationBox.setObjectName("popupDurationBox")
        self.gridLayout_4.addWidget(self.popupDurationBox, 1, 1, 1, 1)
        self.popupOpacityBox = QtWidgets.QDoubleSpinBox(self.groupBox_8)
        self.popupOpacityBox.setDecimals(1)
        self.popupOpacityBox.setMinimum(0.1)
        self.popupOpacityBox.setMaximum(1.0)
        self.popupOpacityBox.setSingleStep(0.1)
        self.popupOpacityBox.setObjectName("popupOpacityBox")
        self.gridLayout_4.addWidget(self.popupOpacityBox, 2, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.groupBox_8)
        self.label_9.setObjectName("label_9")
        self.gridLayout_4.addWidget(self.label_9, 1, 3, 1, 1)
        self.label_23 = QtWidgets.QLabel(self.groupBox_8)
        self.label_23.setObjectName("label_23")
        self.gridLayout_4.addWidget(self.label_23, 1, 0, 1, 1)
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem14, 1, 2, 1, 1)
        self.gridLayout_11.addWidget(self.groupBox_8, 0, 0, 1, 2)
        spacerItem15 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_11.addItem(spacerItem15, 3, 0, 1, 2)
        spacerItem16 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_11.addItem(spacerItem16, 5, 0, 2, 2)
        self.popupEnableBox = QtWidgets.QCheckBox(self.page_5)
        self.popupEnableBox.setObjectName("popupEnableBox")
        self.gridLayout_11.addWidget(self.popupEnableBox, 7, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_5)
        self.gridLayout.addWidget(self.stackedWidget, 0, 2, 1, 1)
        self.gridLayout_8.addLayout(self.gridLayout, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.stackedWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Preferences"))
        __sortingEnabled = self.sideMenuList.isSortingEnabled()
        self.sideMenuList.setSortingEnabled(False)
        item = self.sideMenuList.item(0)
        item.setText(_translate("Dialog", "General"))
        item = self.sideMenuList.item(1)
        item.setText(_translate("Dialog", "Appearance"))
        item = self.sideMenuList.item(2)
        item.setText(_translate("Dialog", "Library viewer"))
        item = self.sideMenuList.item(3)
        item.setText(_translate("Dialog", "Popup"))
        self.sideMenuList.setSortingEnabled(__sortingEnabled)
        self.resetButton.setText(_translate("Dialog", "Reset to default"))
        self.groupBox_4.setTitle(_translate("Dialog", "Music database"))
        self.dbAddButton.setText(_translate("Dialog", "Add"))
        self.dbDeleteButton.setText(_translate("Dialog", "Delete"))
        self.dbResetButton.setToolTip(_translate("Dialog", "Deep database refresh"))
        self.dbResetButton.setText(_translate("Dialog", "..."))
        self.dbTree.headerItem().setText(0, _translate("Dialog", "Library paths"))
        self.dbTree.headerItem().setText(1, _translate("Dialog", "Status"))
        self.groupBox_3.setTitle(_translate("Dialog", "Behavior"))
        self.deleteConfirmBox.setText(_translate("Dialog", "Confirm before deletion"))
        self.resumePlaybackBox.setText(_translate("Dialog", "Resume playback"))
        self.cleanFolderBox.setText(_translate("Dialog", "Remove empty folders"))
        self.label_3.setText(_translate("Dialog", "File manager"))
        self.groupBox_2.setTitle(_translate("Dialog", "Tray icon"))
        self.trayIconBox.setText(_translate("Dialog", "Show tray icon"))
        self.trayMinimizeBox.setText(_translate("Dialog", "Minimize to tray"))
        self.groupBox_10.setTitle(_translate("Dialog", "Icon theme"))
        self.lightThemeRadio.setText(_translate("Dialog", "Light"))
        self.darkThemeRadio.setText(_translate("Dialog", "Dark"))
        self.groupBox_7.setTitle(_translate("Dialog", "Art viewer"))
        self.artTriggerSelectionBox.setText(_translate("Dialog", "Library selection changed"))
        self.label_5.setText(_translate("Dialog", "Viewer size (px)"))
        self.label.setText(_translate("Dialog", "Update triggers"))
        self.artTriggerMediaBox.setText(_translate("Dialog", "Current media changed"))
        self.artShowEmptyBox.setText(_translate("Dialog", "Always show art viewer"))
        self.groupBox_6.setTitle(_translate("Dialog", "Sorting routine"))
        self.label_10.setText(_translate("Dialog", "Available"))
        self.label_4.setText(_translate("Dialog", "Selected"))
        self.sortByDefaultBox.setText(_translate("Dialog", "Enable sorting by default"))
        self.groupBox_5.setTitle(_translate("Dialog", "General"))
        self.stripTitlesBox.setText(_translate("Dialog", "Strip artist / album redundancy from headerless titles"))
        self.label_12.setText(_translate("Dialog", "Window title format"))
        self.label_11.setText(_translate("Dialog", "Notification format"))
        self.expandLibraryBox.setText(_translate("Dialog", "Auto expand library view (filter)"))
        self.label_2.setText(_translate("Dialog", "Tooltip format"))
        self.groupBox.setTitle(_translate("Dialog", "Fonts"))
        self.label_20.setText(_translate("Dialog", "Family"))
        self.label_19.setText(_translate("Dialog", "Size"))
        self.groupBox_9.setTitle(_translate("Dialog", "Geometry"))
        self.label_7.setText(_translate("Dialog", "Vertical offset (px)"))
        self.label_24.setText(_translate("Dialog", "Position"))
        self.label_22.setText(_translate("Dialog", "Width (px)"))
        self.label_25.setText(_translate("Dialog", "Height (px)"))
        self.label_6.setText(_translate("Dialog", "Horizontal offset (px)"))
        self.groupBox_8.setTitle(_translate("Dialog", "Appearance"))
        self.label_21.setText(_translate("Dialog", "Opacity (%)"))
        self.popupBackgroundButton.setToolTip(_translate("Dialog", "Browse background color"))
        self.popupBackgroundButton.setText(_translate("Dialog", "..."))
        self.popupForegroundButton.setToolTip(_translate("Dialog", "Browse foreground color"))
        self.popupForegroundButton.setText(_translate("Dialog", "..."))
        self.label_13.setText(_translate("Dialog", "Foreground"))
        self.label_9.setText(_translate("Dialog", "Background"))
        self.label_23.setText(_translate("Dialog", "Duration (second)"))
        self.popupEnableBox.setText(_translate("Dialog", "Enable popup notifications"))




#!/usr/bin/python3
from PyQt5 import QtWidgets, QtCore, QtMultimedia

try:
    import qoob.backend.parser as parser
except ImportError:
    import backend.parser as parser


class MediaPlayer(QtMultimedia.QMediaPlayer):
    currentMediaChanged = QtCore.pyqtSignal(str)
    setItemColor = QtCore.pyqtSignal(str)
    setSliderRange = QtCore.pyqtSignal(int, int)
    setSliderValue = QtCore.pyqtSignal(int)
    setPlayPauseIcon = QtCore.pyqtSignal(object)
    setShuffleIcon = QtCore.pyqtSignal(object)
    setStatusMessage = QtCore.pyqtSignal(str, str)

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.icon = parent.icon
        self.preferences = parent.preferences
        self.mediaStatusChanged.connect(self._mediaChangedEvent)
        self.durationChanged.connect(self._updateDuration)
        self.positionChanged.connect(self._updateSlider)
        self.seekableChanged.connect(self._seekableEvent)
        self.random = QtCore.QRandomGenerator()
        self.playLater = QtCore.QTimer(singleShot=True, interval=500)
        self.playLater.timeout.connect(self.nextEvent)

        self.setVolume(self.preferences.get("state", "volume"))
        self.lastPlayed = []
        self.lastItem = None
        self.mediaLength = "?"
        self.path = ""

        self.statusTimer = QtCore.QTimer(interval=500)
        self.statusTimer.timeout.connect(self._updateStatus)
        self.statusTimer.start()

    def _currentItem(self):
        return self.parent.tabWidget.current.currentItem()

    def _currentTab(self):
        return self.parent.tabWidget.current

    def _mediaChangedEvent(self, event):
        if event == QtMultimedia.QMediaPlayer.EndOfMedia:
            self.nextEvent()
        elif event == QtMultimedia.QMediaPlayer.InvalidMedia:
            self.setItemColor.emit("red")
            self.setStatusMessage.emit(self.path, "Invalid media")
            self.playLater.start()  # Cannot call nextEvent directly (too fast)

    def _seekableEvent(self):
        if self.resumePlayback and self.isSeekable():
            self.resumePlayback = False
            self.setPosition(self.preferences.get("state", "playback position"))

    def _setCurrentItem(self, item):
        # Slots/signals too slow, sync required
        self._currentTab().setCurrentItem(item)

    def _shuffleNext(self):
        currentRow = self._currentTab().currentIndex().row()
        itemCount = self._currentTab().topLevelItemCount()

        if len(self.lastPlayed) == itemCount:
            self.stopEvent()
            self.lastPlayed = []
        elif itemCount > 1:
            next = self._shuffleTrack(itemCount)
            self._setCurrentItem(next)
            self.activateSelection()
            self.lastPlayed.append(currentRow)

    def _shuffleTrack(self, count):
        next = self.random.bounded(count)
        while next in self.lastPlayed:
            next = self.random.bounded(count)
        return self._currentTab().topLevelItem(next)

    def _updateDuration(self, duration):
        s = duration / 1000
        m, s = divmod(s, 60)
        h, m = divmod(m, 60)
        self.mediaLength = "%02d:%02d:%02d" % (h, m, s)
        self.setSliderRange.emit(0, self.duration())

    def _updateSlider(self, progress):
        if not self.parent.ui.playbackSlider.isSliderDown():
            self.setSliderValue.emit(progress)

    def _updateStatus(self):
        if self.duration() > -1:
            if self.state() == QtMultimedia.QMediaPlayer.PlayingState:
                left = "Now playing: " + self.tags["title"] + " (" + self.tags["artist"] + ")"
            elif self.state() == QtMultimedia.QMediaPlayer.PausedState:
                left = "Paused"
            elif self.state() == QtMultimedia.QMediaPlayer.StoppedState:
                left = "Stopped"

            s = self.parent.ui.playbackSlider.value() / 1000
            m, s = divmod(s, 60)
            h, m = divmod(m, 60)
            elapsed = "%02d:%02d:%02d" % (h, m, s)
            right = f"    {elapsed} / {self.mediaLength}"
            self.setStatusMessage.emit(left, right)

    def activateSelection(self):
        self.resumePlayback = False
        if self.lastItem:
            self.lastItem.setColor("none")

        if self._currentItem():
            self._currentItem().setColor("green")
            self.setCurrentMedia(self._currentItem().text(5))
            self.lastItem = self._currentItem()

        elif self._currentTab().topLevelItemCount() > 0:
            first = self._currentTab().topLevelItem(0)
            self._currentTab().setCurrentItem(first)
            self._setCurrentItem(first)
            self.lastPlayed.append(self._currentTab().currentIndex().row())
            self.activateSelection()

    def clearPlaylist(self):
        self.lastPlayed = []

    def nextEvent(self):
        currentRow = self._currentTab().currentIndex().row()
        itemCount = self._currentTab().topLevelItemCount()
        if self._currentItem():
            if self.shuffle:
                self._shuffleNext()
            elif currentRow == itemCount - 1:
                self.stopEvent()
            else:
                next = self._currentTab().itemBelow(self._currentItem())
                self._setCurrentItem(next)
                self.activateSelection()
                self.lastPlayed.append(currentRow)
        else:
            self.activateSelection()

    def pauseEvent(self):
        self.playLater.stop()
        self.pause()
        self.setPlayPauseIcon.emit(self.icon["play"])

    def playEvent(self):
        self.playLater.stop()
        if self.state() == QtMultimedia.QMediaPlayer.PausedState:
            self.play()
            self.setPlayPauseIcon.emit(self.icon["pause"])

        elif self.state() == QtMultimedia.QMediaPlayer.StoppedState:
            self.activateSelection()

    def playPauseEvent(self):
        self.playLater.stop()
        if self.state() == QtMultimedia.QMediaPlayer.PlayingState:
            self.pauseEvent()
        else:
            self.playEvent()

    def previousEvent(self):
        self.playLater.stop()
        if self._currentItem():
            if self.lastPlayed:
                previous = self.lastPlayed.pop()
                previous = self._currentTab().topLevelItem(previous)
            else:
                previous = self._currentTab().itemAbove(self._currentItem())
                if not previous:
                    previous = self._currentTab().topLevelItem(0)
            self._setCurrentItem(previous)
        self.activateSelection()

    def setCurrentMedia(self, path):
        self.path = path
        self.media = QtMultimedia.QMediaContent(QtCore.QUrl.fromLocalFile(self.path))
        self.setMedia(self.media)
        self.play()
        self.setPlayPauseIcon.emit(self.icon["pause"])
        self.tags = self.parent.parser.header(self.path)
        self.currentMediaChanged.emit(path)
        if "error" in self.tags:
            self.setItemColor.emit("yellow")
            self.setStatusMessage.emit(self.path, self.tags["error"])

    def shuffleEvent(self, checked=False, enable=None):
        self.shuffle = not self.shuffle if enable is None else enable
        if self.shuffle:
            self.setShuffleIcon.emit(self.icon["shuffle_on"])
        else:
            self.setShuffleIcon.emit(self.icon["shuffle_off"])

    def skip(self, seconds):
        self.setPosition(self.position() + seconds*1000)

    def stopEvent(self):
        self.stop()
        self.playLater.stop()
        self.lastPlayed = []
        self.setSliderValue.emit(0)
        self.setPlayPauseIcon.emit(self.icon["play"])



#!/usr/bin/python3
import datetime
import json
import os
import sys
from PyQt5 import QtGui, QtWidgets, QtCore, uic

try:
    import qoob.backend.tools as tools
    import qoob.ui.preferences
except ImportError:
    import backend.tools as tools

# Init common settings
LOCAL_DIR = os.path.dirname(os.path.realpath(__file__)) + '/'
CONFIG_DIR = os.path.expanduser("~/.config/qoob/")
PREFERENCES_FILE = CONFIG_DIR + "preferences.json"

DEFAULT = \
{
    'general':
    {
        'resume playback': True,
        'delete confirm': True,
        'clean folder': True,
        'tray icon': True,
        'tray minimize': True,
        'icon theme': "dark",
        'file manager': 'spacefm --no-saved-tabs',
        'music database': [],
    },
    'art':
    {
        'enable': True,
        'show empty': False,
        'media trigger': True,
        'library trigger': False,
        'size': 200,
    },
    'viewer':
    {
        'expand library': True,
        'strip titles': True,
        'tooltip format': '%title% (%artist%)',
        'title format': '%title% (%artist%) - qoob',
        'notification format': '%artist%\n\'%title%\'',
        'sorting routine': ['Track (Ascending)', 'Album (Ascending)', 'Artist (Ascending)', 'Disable sorting'],
        'sort by default': False,
    },
    'popup':
    {
        'enable': True,
        'media buttons': True,
        'opacity': 0.8,
        'font size': 9,
        'font family': 'Sans Serif',
        'width': 250,
        'height': 75,
        'duration': 3,
        'position': 'bottom right',
        'vertical offset': -15,
        'horizontal offset': -20,
        'foreground': 'white',
        'background': 'black',
    },
    'hotkeys':
    {
        'no modifier':
        {
            'Left': 'skip backward',
            'Right': 'skip forward',
            'Backspace': 'delete from playlist',
            'Del': 'delete from drive',
            'Space': 'play/pause',
        },
        'shift':
        {
            'Backspace': 'add to backlist',
        },
        'ctrl':
        {
            'C': 'copy',
            'X': 'cut',
            'P': 'paste',
            '[': 'playback rate decrease',
            ']': 'playback rate increase',
        },
        'ctrl + shift': {},
        'inhibit': ['Left', 'Right'],
    },
    'actions':
    {
        'tray icon':
        {
            'middle click': 'None',
            'wheel up': 'None',
            'wheel down': 'None'
        },
        'popup':
        {
            'left click': 'None',
            'middle click': 'None',
            'right click': 'None',
        },
    },
    'exec':
    {
        'tray icon':
        {
            'middle click': '',
            'wheel up': '',
            'wheel down': '',
        },
        'popup':
        {
            'left click': '',
            'middle click': '',
            'right click': '',
        },
    },
    'state':
    {
        'volume': 100,
        'playback rate': 1.0,
        'repeat': 0,
        'shuffle': False,
        'playback position': 0,
        'media state': 0,
        'tab': 0,
        'file': '',
        'width': 800,
        'height': 620,
    },
}


class PreferencesDatabase(object):
    def __init__(self, parent):
        self.log = parent.logger.new(name="Preferences")
        if os.path.isfile(PREFERENCES_FILE) and os.stat(PREFERENCES_FILE).st_size > 0:
            self.load()
            if not set(DEFAULT) == set(self.db):
                missing = str(list(filter(lambda x: x not in self.db, DEFAULT)))
                self.log.error(f"Preferences KeyError: {missing}")
                self.log.warning("Restored preferences to default")
                self.db = tools.copyDict(DEFAULT)
                self.save()
        else:
            if not os.path.isdir(CONFIG_DIR):
                os.mkdir(CONFIG_DIR)
            self.db = tools.copyDict(DEFAULT)
            with open(PREFERENCES_FILE, "w") as f:
                f.write(json.dumps(self.db, indent=2, sort_keys=False))
            self.log.info("Created preferences file")

    def get(self, *keys, db=None):
        if not db: db = self.db
        for key in keys:
            try:
                db = db[key]
            except TypeError:
                pass
            except KeyError:
                self.log.error(f"Preferences KeyError: '{key}' not found, loading default")
                default = tools.copyDict(DEFAULT)
                try:
                    for k in keys:
                        default = default[k]
                except KeyError:
                    self.log.critical(f"Preferences KeyError: '{key}' not found in default database")
                    return None
                self.db[keys[0]][keys[1]] = default
                self.save()
                return self.get(*keys)
        return db

    def load(self):
        with open(PREFERENCES_FILE, "r") as f:
            self.db = json.load(f)
        self.log.info("Loaded preferences database")

    def save(self):
        with open(PREFERENCES_FILE, "w") as f:
            f.write(json.dumps(self.db, indent=2, sort_keys=False))
        self.log.info("Saved preferences database")


class PreferencesForm(QtWidgets.QDialog):
    resetMetadata = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__()
        # Load the ui file in case the gui modules are not loaded
        if "qoob.ui.preferences" in sys.modules:
            self.ui = qoob.ui.preferences.Ui_Dialog()
            self.ui.setupUi(self)
        else:
            self.ui = uic.loadUi(LOCAL_DIR + '../ui/preferences.ui', self)
        self.setFixedSize(600, 500)

        self.parent = parent
        self.log = parent.logger.new(name="Preferences")
        self.preferences = parent.preferences
        self.icon = parent.icon
        self._settingsInit()
        self._settingsLoad()

    def closeEvent(self, event):
        event.ignore()
        self.hide()

    def showEvent(self, event):
        self.db = tools.copyDict(self.preferences.db)
        self._settingsLoad()
        event.accept()

    def _dbAdd(self, path):
        listContent = []
        for item in range(self.ui.dbTree.topLevelItemCount()):
            listContent.append(self.ui.dbTree.topLevelItem(item).text(0))
        if os.path.isdir(path) and path not in listContent:
            item = QtWidgets.QTreeWidgetItem()
            item.setText(0, path)
            item.setText(1, "")
            self.ui.dbTree.addTopLevelItem(item)

    def _dbAddButton(self):
        browser = QtWidgets.QFileDialog()
        browser.setFileMode(QtWidgets.QFileDialog.Directory)
        browser.setOption(QtWidgets.QFileDialog.ShowDirsOnly)
        if browser.exec_():
            self._dbAdd(browser.selectedFiles()[0])

    def _dbDeleteButton(self):
        row = self.ui.dbTree.currentIndex().row()
        if row > -1:
            item = self.ui.dbTree.takeTopLevelItem(row)
            path = item.text(0)

    def _dbResetButton(self):
        self.resetMetadata.emit()

    def _popupColorChanged(self, layer):
        color = self.colorWidget.currentColor()
        self.db["parameters"]["popup"][layer] = color.name()

    def _popupPickColor(self, color, layer=None):
        self.colorWidget = QtWidgets.QColorDialog(QtGui.QColor(color))
        self.colorWidget.setWindowFlags(self.colorWidget.windowFlags() | Qt.WindowStaysOnTopHint)
        self.colorWidget.currentColorChanged.connect(lambda: self._popupColorChanged(layer))
        self.colorWidget.exec_()
        return self.colorWidget.selectedColor()

    def _popupPickLayerColor(self, layer):
        currentColor = self.db["parameters"]["popup"][layer]
        color = self._popupPickColor(currentColor, layer)
        if not color.isValid():
            color = QtGui.QColor(currentColor)
        self.db["parameters"]["popup"][layer] = color.name()

    def _settingsInit(self):
        self.db = tools.copyDict(self.preferences.db)
        self.ui.resetButton.clicked.connect(self._settingsReset)
        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.Apply).clicked.connect(self.accepted.emit)
        self.ui.sideMenuList.selectionModel().selectionChanged.connect(self._settingsMenuSelect)

        # Music database
        self.ui.dbAddButton.clicked.connect(self._dbAddButton)
        self.ui.dbDeleteButton.clicked.connect(self._dbDeleteButton)
        self.ui.dbResetButton.clicked.connect(self._dbResetButton)

        # Library viewer
        self.ui.sortAddButton.clicked.connect(self._sortAddButton)
        self.ui.sortRemoveButton.clicked.connect(self._sortRemoveButton)

        # Notifications
        self.ui.popupBackgroundButton.clicked.connect(lambda: self._popupPickLayerColor("background"))
        self.ui.popupForegroundButton.clicked.connect(lambda: self._popupPickLayerColor("foreground"))

    def _settingsLoad(self):
        # General settings
        self.ui.resumePlaybackBox.setChecked(self.db["general"]["resume playback"])
        self.ui.deleteConfirmBox.setChecked(self.db["general"]["delete confirm"])
        self.ui.cleanFolderBox.setChecked(self.db["general"]["clean folder"])
        self.ui.fileManagerLine.setText(self.db["general"]["file manager"])

        # Music database
        self.ui.dbResetButton.setIcon(self.icon["reset"])
        self.ui.dbTree.clear()
        for folder in self.db["general"]["music database"]:
            self._dbAdd(folder)

        # Appearance
        self.ui.lightThemeRadio.setChecked(self.db["general"]["icon theme"] == "light")
        self.ui.darkThemeRadio.setChecked(self.db["general"]["icon theme"] == "dark")
        self.ui.trayIconBox.setChecked(self.db["general"]["tray icon"])
        self.ui.trayMinimizeBox.setChecked(self.db["general"]["tray minimize"])
        self.ui.artShowEmptyBox.setChecked(self.db["art"]["show empty"])
        self.ui.artTriggerMediaBox.setChecked(self.db["art"]["media trigger"])
        self.ui.artTriggerSelectionBox.setChecked(self.db["art"]["library trigger"])
        self.ui.artSizeBox.setValue(self.db["art"]["size"])

        # Library viewer
        self.ui.stripTitlesBox.setChecked(self.db["viewer"]["strip titles"])
        self.ui.expandLibraryBox.setChecked(self.db["viewer"]["expand library"])
        self.ui.sortByDefaultBox.setChecked(self.db["viewer"]["sort by default"])
        self.ui.titleFormatLine.setText(self.db["viewer"]["title format"])
        self.ui.tooltipFormatLine.setText(self.db["viewer"]["tooltip format"])

        notificationFormat = repr(self.db["viewer"]["notification format"])[1:-1]
        notificationFormat = self.db["viewer"]["notification format"].replace("\n", "\\n")
        self.ui.notificationFormatLine.setText(notificationFormat)

        self.ui.sortAvailableList.clear()
        self.ui.sortSelectedList.clear()
        self.ui.sortSelectedList.addItems(self.db["viewer"]["sorting routine"])
        sortActions = ("Artist (Ascending)", "Artist (Descending)", "Album (Ascending)", "Album (Descending)", "Track (Ascending)", "Track (Descending)", "Title (Ascending)", "Title (Descending)", "Duration (Ascending)", "Duration (Descending)", "Disable sorting")
        for item in sortActions:
            if item not in self.db["viewer"]["sorting routine"]:
                self.ui.sortAvailableList.addItem(item)

        # Notifications
        self.ui.popupEnableBox.setChecked(self.db["popup"]["enable"])
        self.ui.popupFontSizeBox.setValue(self.db["popup"]["font size"])
        self.ui.popupFontFamilyCombo.setCurrentText(self.db["popup"]["font family"])
        self.ui.popupOpacityBox.setValue(self.db["popup"]["opacity"])
        self.ui.popupWidthBox.setValue(self.db["popup"]["width"])
        self.ui.popupHeightBox.setValue(self.db["popup"]["height"])
        self.ui.popupDurationBox.setValue(self.db["popup"]["duration"])
        self.ui.popupVOffsetBox.setValue(self.db["popup"]["vertical offset"])
        self.ui.popupHOffsetBox.setValue(self.db["popup"]["horizontal offset"])

        position = self.db["popup"]["position"]
        if position == "top left":
            self.ui.topLeftRadio.setChecked(True)
        elif position == "top center":
            self.ui.topCenterRadio.setChecked(True)
        elif position == "top right":
            self.ui.topRightRadio.setChecked(True)
        elif position == "middle left":
            self.ui.middleLeftRadio.setChecked(True)
        elif position == "middle center":
            self.ui.middleCenterRadio.setChecked(True)
        elif position == "middle right":
            self.ui.middleRightRadio.setChecked(True)
        elif position == "bottom left":
            self.ui.bottomLeftRadio.setChecked(True)
        elif position == "bottom center":
            self.ui.bottomCenterRadio.setChecked(True)
        elif position == "bottom right":
            self.ui.bottomRightRadio.setChecked(True)

    def _settingsMenuSelect(self):
        index = self.ui.sideMenuList.currentRow()
        self.ui.stackedWidget.setCurrentIndex(index)

    def _settingsReset(self):
        self.db = tools.copyDict(DEFAULT)
        self._settingsLoad()

    def _sortAddButton(self):
        item = self.ui.sortAvailableList.currentItem()
        if item:
            row = self.ui.sortAvailableList.currentRow()
            item = self.ui.sortAvailableList.takeItem(row)
            self.ui.sortSelectedList.addItem(item)

    def _sortRemoveButton(self):
        row = self.ui.sortSelectedList.currentRow()
        item = self.ui.sortSelectedList.takeItem(row)
        if item:
            self.ui.sortAvailableList.addItem(item)

    @QtCore.pyqtSlot(str)
    def dbDoneSlot(self, path):
        for folder in range(self.ui.dbTree.topLevelItemCount()):
            item =  self.ui.dbTree.topLevelItem(folder)
            if item.text(0) == path:
                now = datetime.datetime.now()
                item.setText(1, f"Done ({now.day}-{now.strftime('%b')} {now.strftime('%H:%M:%S')})")

    @QtCore.pyqtSlot(str)
    def dbStartSlot(self, path):
        for folder in range(self.ui.dbTree.topLevelItemCount()):
            item =  self.ui.dbTree.topLevelItem(folder)
            if item.text(0) == path:
                item.setText(1, "Scanning")

    @QtCore.pyqtSlot()
    def setPending(self):
        for folder in range(self.ui.dbTree.topLevelItemCount()):
            item =  self.ui.dbTree.topLevelItem(folder)
            item.setText(1, "Pending")

    @QtCore.pyqtSlot()
    def settingsSave(self):
        # General Settings
        self.db["general"]["resume playback"] = self.ui.resumePlaybackBox.isChecked()
        self.db["general"]["delete confirm"] = self.ui.deleteConfirmBox.isChecked()
        self.db["general"]["clean folder"] = self.ui.cleanFolderBox.isChecked()
        self.db["general"]["file manager"] = self.ui.fileManagerLine.text()

        musicDatabase = []
        for item in range(self.ui.dbTree.topLevelItemCount()):
            item = self.ui.dbTree.topLevelItem(item)
            musicDatabase.append(item.text(0))
        self.db["general"]["music database"] = musicDatabase

        # Appearance
        if self.ui.lightThemeRadio.isChecked():
            self.db["general"]["icon theme"] = "light"
        elif self.ui.darkThemeRadio.isChecked():
            self.db["general"]["icon theme"] = "dark"
        self.db["general"]["tray icon"] = self.ui.trayIconBox.isChecked()
        self.db["general"]["tray minimize"] = self.ui.trayMinimizeBox.isChecked()
        self.db["art"]["show empty"] = self.ui.artShowEmptyBox.isChecked()
        self.db["art"]["media trigger"] = self.ui.artTriggerMediaBox.isChecked()
        self.db["art"]["library trigger"] = self.ui.artTriggerSelectionBox.isChecked()
        self.db["art"]["size"] = self.ui.artSizeBox.value()

        # Library viewer
        self.db["viewer"]["strip titles"] = self.ui.stripTitlesBox.isChecked()
        self.db["viewer"]["expand library"] = self.ui.expandLibraryBox.isChecked()
        self.db["viewer"]["title format"] = self.ui.titleFormatLine.text()
        self.db["viewer"]["tooltip format"] = self.ui.tooltipFormatLine.text()
        self.db["viewer"]["notification format"] = self.ui.notificationFormatLine.text().replace("\\n","\n")
        self.db["viewer"]["sort by default"] = self.ui.sortByDefaultBox.isChecked()
        sortingRoutine = []
        for item in range(self.ui.sortSelectedList.count()):
            sortingRoutine.append(self.ui.sortSelectedList.item(item).text())
        self.db["viewer"]["sorting routine"] = sortingRoutine

        # Notifications
        self.db["popup"]["enable"] = self.ui.popupEnableBox.isChecked()
        self.db["popup"]["font size"] = self.ui.popupFontSizeBox.value()
        self.db["popup"]["font family"] = self.ui.popupFontFamilyCombo.currentText()
        self.db["popup"]["opacity"] = self.ui.popupOpacityBox.value()
        self.db["popup"]["width"] = self.ui.popupWidthBox.value()
        self.db["popup"]["height"] = self.ui.popupHeightBox.value()
        self.db["popup"]["duration"] = self.ui.popupDurationBox.value()
        self.db["popup"]["vertical offset"] = self.ui.popupVOffsetBox.value()
        self.db["popup"]["horizontal offset"] = self.ui.popupHOffsetBox.value()

        if self.ui.topLeftRadio.isChecked():
            position = "top left"
        elif self.ui.topCenterRadio.isChecked():
            position = "top center"
        elif self.ui.topRightRadio.isChecked():
            position = "top right"
        elif self.ui.middleLeftRadio.isChecked():
            position = "middle left"
        elif self.ui.middleCenterRadio.isChecked():
            position = "middle center"
        elif self.ui.middleRightRadio.isChecked():
            position = "middle right"
        elif self.ui.bottomLeftRadio.isChecked():
            position = "bottom left"
        elif self.ui.bottomCenterRadio.isChecked():
            position = "bottom center"
        elif self.ui.bottomRightRadio.isChecked():
            position = "bottom right"
        self.db["popup"]["position"] = position



#!/usr/bin/python3
from PyQt5 import QtDBus
import sys


def main():
    bus = QtDBus.QDBusConnection.sessionBus()
    interface = QtDBus.QDBusInterface("org.qoob.session", "/org/qoob/session", "org.qoob.session", bus)
    cmd = "%".join(str(arg) for arg in sys.argv[1:])

    # Pass the arguments to the existing bus
    if interface.isValid():
        interface.call("parse", cmd)
        sys.exit(0)

    # Create a new instance
    else:
        try:
            import qoob.main as qoob
        except ImportError:
            import main as qoob
        qoob.main(cmd)

if __name__ == '__main__':
    main()



#!/usr/bin/python3
import os
import re
import sys

from PyQt5 import QtWidgets, QtCore, QtGui
import mutagen

try:
    import qoob.backend.tools as tools
except ImportError:
    import backend.tools as tools

ALLOWED_AUDIO_TYPES = (".mp3", ".flac", ".ogg", ".m4a", ".wav")
ALLOWED_ART_TYPES = (".jpg", ".jpeg", ".png")
COMMON_ART_NAMES = ("front", "cover", "folder", "outside")

mappings = \
{
    "artist": ("TPE0", "TPE1", "TPE2", u"©ART", "Author", "Artist", "ARTIST", "artist", "TRACK ARTIST", "TRACKARTIST", "TrackArtist", "Track Artist"),
    "album": ("TALB", "ALBUM", "Album", u"©alb", "album"),
    "track": ("TRCK", "TRACKNUMBER", "Track", "trkn", "tracknumber"),
    "title": ("TIT2", "TITLE", "Title", u"©nam", "title"),
}

def allowedType(path):
    ext = os.path.splitext(path)[1].lower()
    return (ext in ALLOWED_AUDIO_TYPES)


class Parser(QtCore.QObject):
    setFilterEnable = QtCore.pyqtSignal(bool)
    addAlbum = QtCore.pyqtSignal(object, str)
    addArtist = QtCore.pyqtSignal(object)
    startDb = QtCore.pyqtSignal(str)
    doneDb = QtCore.pyqtSignal(str)
    pending = QtCore.pyqtSignal()
    done = QtCore.pyqtSignal()
    clear = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.log = parent.logger.new("Parser")
        self.preferences = parent.preferences
        self.metadata = tools.Database("metadata")
        self.trackPattern = re.compile(r"\b\d{2}\b|\b\d{2}(?=\D)")

    def _cleanMetadata(self):
        for path in dict(self.metadata.db):
            if not os.path.exists(path):
                del self.metadata.db[path]

    def _fillHeader(self, header, tags):
        title, artist, album, track = tags

        if "artist" not in header:
            header["artist"] = artist

        if "album" not in header:
            header["album"] = album

        if "track" not in header:
            header["track"] = track

        if "title" not in header:
            if self.preferences.get("viewer", "strip titles"):
                title = re.sub(r"\W*" + artist + r"\W*", "", title, flags=re.IGNORECASE, count=1)
                title = re.sub(r"\W*" + album + r"\W*", "", title, flags=re.IGNORECASE, count=1)
                title = re.sub(r"\W*" + track + r"\W*", self._trackChar, title, flags=re.IGNORECASE, count=1)
                header["title"] = title
            else:
                header["title"] = re.sub(r"\W*" + track + r"\W*", self._trackChar, title, flags=re.IGNORECASE, count=1)
        return header

    def _guessFileHeader(self, path):
        basename = os.path.basename(path)
        title = os.path.splitext(basename)[0]

        # Fetch album and artist from path
        album = os.path.abspath(os.path.join(path, os.pardir))
        artist = os.path.abspath(os.path.join(album, os.pardir))
        album = os.path.basename(album)
        artist = os.path.basename(artist)

        # Fetch track number from title,
        # remove artist/album in case it contain two digits numbers
        track = re.sub(r"\W*" + artist + r"\W*", "", title, flags=re.IGNORECASE, count=1)
        track = re.sub(r"\W*" + album + r"\W*", "", track, flags=re.IGNORECASE, count=1)
        track = re.search(self.trackPattern, track)
        track = track.group() if track else ""
        return (title, artist, album, track)

    def _hasAudioFiles(self, path):
        for root, subfolders, files in os.walk(path):
            for f in files:
                if allowedType(f):
                    return True
        return False

    def _item(self, path, node):
        name = os.path.splitext(os.path.basename(path))[0]
        item = QtWidgets.QTreeWidgetItem()
        item.setText(0, name)
        item.setText(1, path)
        return item

    def _parseHeaderTag(self, header, key, tag):
        if isinstance(header, mutagen.flac.FLAC) or isinstance(header, mutagen.oggvorbis.OggVorbis):
            header = dict(header)

        header = header[tag][-1]
        if isinstance(header, tuple):
            header = str(header[0])

        if key == "track":
            return header.split("/")[0]
        return header

    def _readHeader(self, path):
        tags = {"duration": "?"}
        header = None
        try:
            header = mutagen.File(path)
        except:
            tags["error"] = f"{sys.exc_info()[0]} {sys.exc_info()[1]}"
            self.log.warning(f"{path}\n{sys.exc_info()[0]} {sys.exc_info()[1]}\n")

        if header:
            if header.info:
                s = int(header.info.length)
                m, s = divmod(s, 60)
                h, m = divmod(m, 60)
                tags["duration"] = "%02d:%02d:%02d" % (h, m, s)

            for key in ("artist", "album", "track", "title"):
                for tag in mappings[key]:
                    try:
                        tags[key] = self._parseHeaderTag(header, key, tag)
                        break
                    except KeyError:
                        pass
        return tags

    def _recursiveScan(self, folder, node=None):
        try:
            for f in os.listdir(folder):
                path = os.path.join(folder, f)

                if os.path.isdir(path):
                    if self._hasAudioFiles(path):
                        item = self._item(path, node=node)
                        if node:
                            self.addAlbum.emit(node, path)
                        else:
                            self.addArtist.emit(item)
                        self._recursiveScan(path, node=item)

                elif allowedType(path):
                    if path not in self.metadata.db:
                        self.metadata.db[path] = self.header(path)
        except PermissionError:
            pass

    def _trackChar(self, match):
        return "" if match.start() == 0 else " - "

    @QtCore.pyqtSlot(str)
    def art(self, path):
        images = []
        if not os.path.isdir(path):
            path = os.path.abspath(os.path.join(path, os.pardir))
        for f in os.listdir(path):
            if os.path.splitext(f)[1].lower() in ALLOWED_ART_TYPES:
                images.append(f)
        for name in images:
            for wanted in COMMON_ART_NAMES:
                if name.lower().count(wanted.lower()) > 0:
                    return QtGui.QPixmap(f"{path}/{name}")
        if images:
            return QtGui.QPixmap(f"{path}/{images[0]}")
        return QtGui.QPixmap(None)

    @QtCore.pyqtSlot(str)
    def header(self, path):
        if path in self.metadata.db:
            return self.metadata.db[path]

        header = self._readHeader(path)
        if set({"artist", "album", "track", "title"}) - set(header):
            tags = self._guessFileHeader(path)
            header = self._fillHeader(header, tags)
        return header

    @QtCore.pyqtSlot(str)
    def removeDatabase(self, folder):
        for path in dict(self.metadata.db):
            if path.startswith(folder):
                del self.metadata.db[path]
        self.scanAll()

    @QtCore.pyqtSlot()
    def reset(self):
        self.metadata.db = {}
        self.scanAll()

    @QtCore.pyqtSlot()
    def scanAll(self):
        self.setFilterEnable.emit(False)
        self.clear.emit()
        self.pending.emit()
        self._cleanMetadata()
        for folder in list(self.preferences.get("general", "music database")):
            if os.path.isdir(folder):
                self.startDb.emit(folder)
                self._recursiveScan(folder)
                self.doneDb.emit(folder)
                self.metadata.save()
            else:
                self.log.error(f"Music folder '{folder}' not found")
                self.preferences.db["general"]["music database"].remove(folder)
                self.preferences.save()
        self.setFilterEnable.emit(True)
        self.done.emit()




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qoob/ui/player.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 620)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.tabLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.tabLayout.setContentsMargins(10, 10, 10, 10)
        self.tabLayout.setObjectName("tabLayout")
        self.shuffleButton = QtWidgets.QToolButton(self.centralwidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("icons/shuffle_off.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.shuffleButton.setIcon(icon)
        self.shuffleButton.setObjectName("shuffleButton")
        self.tabLayout.addWidget(self.shuffleButton, 0, 7, 1, 1)
        self.playButton = QtWidgets.QToolButton(self.centralwidget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("icons/play.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.playButton.setIcon(icon1)
        self.playButton.setObjectName("playButton")
        self.tabLayout.addWidget(self.playButton, 0, 4, 1, 1)
        self.previousButton = QtWidgets.QToolButton(self.centralwidget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("icons/previous.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.previousButton.setIcon(icon2)
        self.previousButton.setObjectName("previousButton")
        self.tabLayout.addWidget(self.previousButton, 0, 5, 1, 1)
        self.libraryFrame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.libraryFrame.sizePolicy().hasHeightForWidth())
        self.libraryFrame.setSizePolicy(sizePolicy)
        self.libraryFrame.setMinimumSize(QtCore.QSize(300, 0))
        self.libraryFrame.setMaximumSize(QtCore.QSize(300, 16777215))
        self.libraryFrame.setLineWidth(0)
        self.libraryFrame.setObjectName("libraryFrame")
        self.libraryLayout = QtWidgets.QVBoxLayout(self.libraryFrame)
        self.libraryLayout.setContentsMargins(-1, 0, -1, 0)
        self.libraryLayout.setObjectName("libraryLayout")
        self.artFrame = QtWidgets.QFrame(self.libraryFrame)
        self.artFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.artFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.artFrame.setObjectName("artFrame")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.artFrame)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.artLabel = QtWidgets.QLabel(self.artFrame)
        self.artLabel.setMaximumSize(QtCore.QSize(280, 280))
        self.artLabel.setText("")
        self.artLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.artLabel.setObjectName("artLabel")
        self.horizontalLayout.addWidget(self.artLabel)
        self.libraryLayout.addWidget(self.artFrame)
        self.frame = QtWidgets.QFrame(self.libraryFrame)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setObjectName("frame")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setMaximumSize(QtCore.QSize(70, 16777215))
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)
        self.libraryFilterLine = QtWidgets.QLineEdit(self.frame)
        self.libraryFilterLine.setObjectName("libraryFilterLine")
        self.gridLayout_2.addWidget(self.libraryFilterLine, 0, 1, 1, 1)
        self.libraryFilterClearButton = QtWidgets.QPushButton(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.libraryFilterClearButton.sizePolicy().hasHeightForWidth())
        self.libraryFilterClearButton.setSizePolicy(sizePolicy)
        self.libraryFilterClearButton.setMaximumSize(QtCore.QSize(16, 16))
        self.libraryFilterClearButton.setStyleSheet("")
        self.libraryFilterClearButton.setText("")
        self.libraryFilterClearButton.setObjectName("libraryFilterClearButton")
        self.gridLayout_2.addWidget(self.libraryFilterClearButton, 0, 2, 1, 1)
        self.libraryLayout.addWidget(self.frame)
        self.frame_2 = QtWidgets.QFrame(self.libraryFrame)
        self.frame_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_2.setLineWidth(0)
        self.frame_2.setObjectName("frame_2")
        self.gridLayout = QtWidgets.QGridLayout(self.frame_2)
        self.gridLayout.setContentsMargins(0, 7, 0, 7)
        self.gridLayout.setObjectName("gridLayout")
        self.preferencesButton = QtWidgets.QToolButton(self.frame_2)
        self.preferencesButton.setObjectName("preferencesButton")
        self.gridLayout.addWidget(self.preferencesButton, 0, 0, 1, 1)
        self.collapseButton = QtWidgets.QToolButton(self.frame_2)
        self.collapseButton.setObjectName("collapseButton")
        self.gridLayout.addWidget(self.collapseButton, 0, 6, 1, 1)
        self.expandButton = QtWidgets.QToolButton(self.frame_2)
        self.expandButton.setObjectName("expandButton")
        self.gridLayout.addWidget(self.expandButton, 0, 7, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 3, 1, 1)
        self.artButton = QtWidgets.QToolButton(self.frame_2)
        self.artButton.setObjectName("artButton")
        self.gridLayout.addWidget(self.artButton, 0, 2, 1, 1)
        self.refreshButton = QtWidgets.QToolButton(self.frame_2)
        self.refreshButton.setObjectName("refreshButton")
        self.gridLayout.addWidget(self.refreshButton, 0, 1, 1, 1)
        self.libraryLayout.addWidget(self.frame_2)
        self.tabLayout.addWidget(self.libraryFrame, 1, 0, 1, 1)
        self.nextButton = QtWidgets.QToolButton(self.centralwidget)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("icons/next.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.nextButton.setIcon(icon3)
        self.nextButton.setObjectName("nextButton")
        self.tabLayout.addWidget(self.nextButton, 0, 6, 1, 1)
        self.stopButton = QtWidgets.QToolButton(self.centralwidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap("icons/stop.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stopButton.setIcon(icon4)
        self.stopButton.setObjectName("stopButton")
        self.tabLayout.addWidget(self.stopButton, 0, 3, 1, 1)
        self.playbackSlider = QtWidgets.QSlider(self.centralwidget)
        self.playbackSlider.setOrientation(QtCore.Qt.Horizontal)
        self.playbackSlider.setObjectName("playbackSlider")
        self.tabLayout.addWidget(self.playbackSlider, 0, 0, 1, 2)
        self.volumeSlider = QtWidgets.QSlider(self.centralwidget)
        self.volumeSlider.setMaximumSize(QtCore.QSize(100, 16777215))
        self.volumeSlider.setMaximum(100)
        self.volumeSlider.setOrientation(QtCore.Qt.Horizontal)
        self.volumeSlider.setTickPosition(QtWidgets.QSlider.TicksBothSides)
        self.volumeSlider.setTickInterval(50)
        self.volumeSlider.setObjectName("volumeSlider")
        self.tabLayout.addWidget(self.volumeSlider, 0, 2, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 800, 20))
        self.menuBar.setObjectName("menuBar")
        MainWindow.setMenuBar(self.menuBar)
        self.actionPreferences = QtWidgets.QAction(MainWindow)
        self.actionPreferences.setObjectName("actionPreferences")

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.shuffleButton.setToolTip(_translate("MainWindow", "Shuffle"))
        self.shuffleButton.setText(_translate("MainWindow", "..."))
        self.playButton.setToolTip(_translate("MainWindow", "Play / Pause"))
        self.playButton.setText(_translate("MainWindow", "..."))
        self.previousButton.setToolTip(_translate("MainWindow", "Previous"))
        self.previousButton.setText(_translate("MainWindow", "..."))
        self.label.setText(_translate("MainWindow", "Filter"))
        self.libraryFilterClearButton.setToolTip(_translate("MainWindow", "Clear filter"))
        self.preferencesButton.setToolTip(_translate("MainWindow", "Preferences"))
        self.preferencesButton.setText(_translate("MainWindow", "..."))
        self.collapseButton.setToolTip(_translate("MainWindow", "Collapse all"))
        self.collapseButton.setText(_translate("MainWindow", "..."))
        self.expandButton.setToolTip(_translate("MainWindow", "Expand all"))
        self.expandButton.setText(_translate("MainWindow", "..."))
        self.artButton.setToolTip(_translate("MainWindow", "Art viewer"))
        self.artButton.setText(_translate("MainWindow", "..."))
        self.refreshButton.setToolTip(_translate("MainWindow", "Quick database refresh"))
        self.refreshButton.setText(_translate("MainWindow", "..."))
        self.nextButton.setToolTip(_translate("MainWindow", "Next"))
        self.nextButton.setText(_translate("MainWindow", "..."))
        self.stopButton.setToolTip(_translate("MainWindow", "Stop"))
        self.stopButton.setText(_translate("MainWindow", "..."))
        self.actionPreferences.setText(_translate("MainWindow", "Preferences"))




#!/usr/bin/python3
import os
import sys
from contextlib import suppress
from PyQt5 import QtGui, QtWidgets, QtCore, QtMultimedia, uic

try:
    import qoob.backend.tools as tools
    import qoob.backend.player as player
    import qoob.backend.parser as parser
    import qoob.frontend.preferences as preferences
    import qoob.frontend.trinkets as trinkets
    import qoob.frontend.playlist as playlist
    import qoob.ui.player
except ImportError:
    import backend.tools as tools
    import backend.player as player
    import backend.parser as parser
    import frontend.preferences as preferences
    import frontend.trinkets as trinkets
    import frontend.playlist as playlist

LOCAL_DIR = os.path.dirname(os.path.realpath(__file__)) + "/"
DB_DIR = os.path.expanduser("~/.config/qoob/")


class LibraryTree(QtWidgets.QTreeWidget):
    selectedItem = QtCore.pyqtSignal(str)
    clearFilter = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.preferences = parent.preferences
        self.libraryView = parent.tabWidget.tabs["Library viewer"]["playlist"]
        self.setAlternatingRowColors(True)
        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.setIndentation(10)
        self.header().setVisible(False)
        self.itemSelectionChanged.connect(self._selectItem)

    def _hasFilteredFiles(self, path):
        for root, subfolders, files in os.walk(path):
            for f in files:
                if parser.allowedType(f):
                    if self._sift(f):
                        return True
        return False

    def _recursiveScan(self, folder, node=None):
        try:
            for f in os.listdir(folder):
                path = os.path.join(folder, f)
                if os.path.isdir(path):
                    if self._hasFilteredFiles(path):
                        item = self.addItem(node, path)
                        self._recursiveScan(path, node=item)

                elif parser.allowedType(path):
                    if self._sift(f):
                        self.addItem(node, path)
        except PermissionError:
            pass

    def _scanSelection(self, path):
        for root, subfolder, files in os.walk(path):
            for f in files:
                if self._sift(f):
                    self.libraryView.add(os.path.join(root, f))
        if os.path.isfile(path):
            if self._sift(path):
                self.libraryView.add(path)

    def _selectItem(self):
        self.libraryView.clear()
        for item in self.selectedItems():
            if item.text(1):
                self._scanSelection(item.text(1))
            elif item.text(0) == "All Music":
                for folder in self.preferences.get("general", "music database"):
                    self._scanSelection(folder)
            self.selectedItem.emit(item.text(1))

    def _sift(self, string):
        sift = self.parent.ui.libraryFilterLine.text()
        return string.lower().count(sift.lower()) > 0

    def addArtist(self, node):
        self.addTopLevelItem(node)
        self.sortItems(0, QtCore.Qt.AscendingOrder)

    def addItem(self, node, path):
        name = os.path.splitext(os.path.basename(path))[0]
        item = QtWidgets.QTreeWidgetItem()
        item.setText(0, name)
        item.setText(1, path)
        try:
            if node:
                node.addChild(item)
            else:
                self.addTopLevelItem(item)
        except RuntimeError:
            pass
        return item

    def addWildCard(self):
        item = QtWidgets.QTreeWidgetItem()
        item.setText(0, "All Music")
        self.insertTopLevelItem(0, item)

    def clearAll(self):
        self.clearFilter.emit()
        self.clear()

    """
    def deleteItem(self, item, path):
        for subItem in range(item.childCount()):
            subItem = item.child(subItem)
            if subItem.childCount() > 0:
                self.deleteItem(subItem, path)
            elif subItem.text(1) == path:
                index = item.indexOfChild(subItem)
                item.takeChild(index)
                return True
        return False
    """

    def filterEvent(self):
        self.refresh()
        if self.preferences.get("viewer", "expand library"):
            count = self.topLevelItemCount()
            for item in range(self.topLevelItemCount()):
                count += self.topLevelItem(item).childCount()
            if (count * 12) <= self.height():  # Where 12px is the height of one item
                self.expandAll()
        self.sortItems(0, QtCore.Qt.AscendingOrder)
        self.addWildCard()

    def refresh(self):
        self.clear()
        for folder in self.preferences.get("general", "music database"):
            self._recursiveScan(folder)


class Main(QtWidgets.QMainWindow):
    def __init__(self, parent):
        super().__init__()
        if "qoob.ui.player" in sys.modules:
            self.ui = qoob.ui.player.Ui_MainWindow()
            self.ui.setupUi(self)
        else:
            self.ui = uic.loadUi(LOCAL_DIR + "ui/player.ui", self)

        self.copied = []
        self.parent = parent
        self.logger = tools.Logger(name="qoob", path=DB_DIR)
        self.log = self.logger.new("qoob")
        self.preferences = preferences.PreferencesDatabase(self)
        self._initStyle()
        self.preferencesForm = preferences.PreferencesForm(self)
        self.popup = trinkets.PopupWidget(self)
        self.process = QtCore.QProcess()
        self.random = QtCore.QRandomGenerator()
        if not self.preferences.db["general"]["music database"]:
            self._findDefaultMusicFolder()

        self.player = player.MediaPlayer(self)
        self.parser = parser.Parser(self)
        self.tabWidget = playlist.Tabs(self)
        self.library = LibraryTree(self)
        self.trayIcon = trinkets.TrayIcon(self)

        self.player.currentMediaChanged.connect(self.currentMediaChanged)
        self.player.setItemColor.connect(self.setItemColor)
        self.player.setSliderRange.connect(self.ui.playbackSlider.setRange)
        self.player.setSliderValue.connect(self.ui.playbackSlider.setValue)
        self.player.setPlayPauseIcon.connect(self.ui.playButton.setIcon)
        self.player.setShuffleIcon.connect(self.ui.shuffleButton.setIcon)
        self.player.setStatusMessage.connect(self.setStatusMessage)
        self.player.shuffle = self.preferences.get("state", "shuffle")
        self.ui.volumeSlider.setValue(self.player.volume())

        self.tabWidget.keyEvent.connect(self.parseKey)
        self.tabWidget.changed.connect(self.player.clearPlaylist)
        self.library.selectedItem.connect(self.librarySelect)
        self.library.clearFilter.connect(self.ui.libraryFilterLine.clear)
        self.preferencesForm.accepted.connect(self.preferencesSave)
        self.preferencesForm.resetMetadata.connect(self.parser.reset)
        self.preferencesForm.activateWindow()

        self.parserThread = QtCore.QThread()
        self.parserThread.started.connect(self.parser.scanAll)
        self.parser.moveToThread(self.parserThread)
        self.parser.done.connect(self.libraryDone)
        self.parser.addArtist.connect(self.library.addArtist)
        self.parser.addAlbum.connect(self.library.addItem)
        self.parser.clear.connect(self.library.clearAll)
        self.parser.startDb.connect(self.preferencesForm.dbStartSlot)
        self.parser.doneDb.connect(self.preferencesForm.dbDoneSlot)
        self.parser.pending.connect(self.preferencesForm.setPending)
        self.parser.setFilterEnable.connect(self.ui.libraryFilterLine.setEnabled)
        self.parserThread.start()

        self.trayIcon.hideWindow.connect(self.hide)
        self.trayIcon.showWindow.connect(self.show)
        self.trayIcon.close.connect(self.close)
        self.trayIcon.play.connect(self.player.playPauseEvent)
        self.trayIcon.stop.connect(self.player.stopEvent)
        self.trayIcon.next.connect(self.player.nextEvent)
        self.trayIcon.previous.connect(self.player.previousEvent)
        self.trayIcon.shuffle.connect(self.player.shuffleEvent)
        self.trayIcon.showPreferencesForm.connect(self.preferencesForm.show)
        self.trayIcon.popup.connect(self.popup.display)

        self.menu = QtWidgets.QMenu()
        self.menu.aboutToShow.connect(self._menuRefresh)

        self.ui.statusBar = QtWidgets.QStatusBar()
        self.ui.statusRightLabel = QtWidgets.QLabel()

        self.ui.statusBar.addPermanentWidget(self.ui.statusRightLabel)
        self.ui.libraryLayout.insertWidget(1, self.library)
        self.ui.tabLayout.addWidget(self.tabWidget, 1, 1, 1, 7)

        self.ui.playButton.clicked.connect(self.player.playPauseEvent)
        self.ui.stopButton.clicked.connect(self.player.stopEvent)
        self.ui.nextButton.clicked.connect(self.player.nextEvent)
        self.ui.previousButton.clicked.connect(self.player.previousEvent)
        self.ui.shuffleButton.clicked.connect(self.player.shuffleEvent)
        self.ui.preferencesButton.clicked.connect(self.preferencesShow)
        self.ui.refreshButton.clicked.connect(self.parser.scanAll)
        self.ui.collapseButton.clicked.connect(self.library.collapseAll)
        self.ui.expandButton.clicked.connect(self.library.expandAll)
        self.ui.libraryFilterLine.textChanged.connect(self.library.filterEvent)
        self.ui.libraryFilterClearButton.clicked.connect(self.ui.libraryFilterLine.clear)
        self.ui.volumeSlider.valueChanged.connect(self.player.setVolume)
        self.ui.playbackSlider.installEventFilter(self)
        self.ui.volumeSlider.installEventFilter(self)
        self.ui.artButton.clicked.connect(self._artViewerToggle)

        self._artInit()
        self.setStatusBar(self.ui.statusBar)
        self.setWindowTitle("qoob")
        self.setWindowIcon(self.icon["qoob"])
        width = self.preferences.get("state", "width")
        height = self.preferences.get("state", "height")
        self.resize(width, height)
        self.tabWidget.setFocus(True)
        self._resumePlayback()
        self.show()

    def closeEvent(self, event):
        self.preferences.db["state"]["shuffle"] = self.player.shuffle
        self.preferences.db["state"]["volume"] = self.player.volume()
        self.preferences.db["state"]["tab"] = self.tabWidget.tabBar().currentIndex()
        self.preferences.db["state"]["playback position"] = self.player.position()
        self.preferences.db["state"]["media state"] = self.player.state()
        self.preferences.db["state"]["file"] = self.player.path
        self.preferences.db["state"]["width"] = self.width()
        self.preferences.db["state"]["height"] = self.height()
        self.preferences.save()
        self.tabWidget.playlistSave()
        self.parent.exit()

    def eventFilter(self, obj, event):
        if event.type() == QtCore.QEvent.MouseButtonRelease:
            mouseEvent = QtGui.QMouseEvent(event)
            position = QtWidgets.QStyle.sliderValueFromPosition(obj.minimum(), obj.maximum(), mouseEvent.x(), obj.width())
            obj.setValue(position)
            if obj == self.ui.playbackSlider:
                self.player.setPosition(obj.value())
        return QtCore.QObject.event(obj, event)

    def changeEvent(self, event):
        # Override minimize event
        if event.type() == QtCore.QEvent.WindowStateChange:
            if self.windowState() & QtCore.Qt.WindowMinimized:
                if self.preferences.get("general", "tray minimize") and self.preferences.get("general", "tray icon"):
                    self.setWindowState(QtCore.Qt.WindowNoState)
                    self.hide() if self.isVisible() else self.show()

    def _artDisplay(self):
        if self.preferences.get("art", "enable"):
            size = self.preferences.get("art", "size")
            if self.art.isNull():
                art = self.icon["qoob"].pixmap(64, 64)
                self.ui.artFrame.setVisible(self.preferences.get("art", "show empty"))
            else:
                aspectRatio = QtCore.Qt.IgnoreAspectRatio
                smooth = QtCore.Qt.SmoothTransformation
                art = self.art.scaled(size, size, aspectRatio, transformMode=smooth)
                self.ui.artFrame.setVisible(True)
            self.ui.artLabel.setFixedSize(size, size)
            self.ui.artLabel.setPixmap(art)

    def _artInit(self):
        size = self.preferences.get("art", "size")
        art = self.icon["qoob"].pixmap(size, size)
        self.ui.artLabel.setPixmap(art)
        self.ui.artFrame.setVisible(self.preferences.get("art", "show empty") and self.preferences.get("art", "enable"))
        if self.preferences.get("art", "enable"):
            self.ui.artButton.setIcon(self.icon["art_on"])
        else:
            self.ui.artButton.setIcon(self.icon["art_off"])

    def _artLoad(self, path):
        self.art = self.parser.art(path)

    def _artViewerToggle(self):
        if self.preferences.get("art", "enable"):
            self.preferences.db["art"]["enable"] = False
            self.ui.artButton.setIcon(self.icon["art_off"])
            self.ui.artFrame.setVisible(False)
        else:
            self.preferences.db["art"]["enable"] = True
            self.ui.artButton.setIcon(self.icon["art_on"])
            self._artDisplay()
        self.preferences.save()

    def _delete(self, files):
        self.action("delete")
        for f in files:
            if os.path.isfile(f):
                os.remove(f)

            # Remove empty folder
            if self.preferences.get("general", "clean folder"):
                parent = os.path.abspath(os.path.join(f, os.pardir))
                if len(os.listdir(parent)) == 0:
                    os.rmdir(parent)
                    self.library.refresh()
                    self.library.addWildCard()
                    return

            """
            # Remove item from library tree
            for item in range(self.library.topLevelItemCount()):
                item = self.library.topLevelItem(item)
                if self.library.deleteItem(item, f):
                    break
            """

    def _deletePrompt(self, files):
        if self.preferences.get("general", "delete confirm"):
            names = ""
            for f in files:
                names += "\n" + os.path.basename(f)
            msg = QtWidgets.QMessageBox()
            msg.setWindowFlags(msg.windowFlags() | QtCore.Qt.WindowStaysOnTopHint)
            msg.setIcon(QtWidgets.QMessageBox.Warning)
            msg.setWindowTitle("Delete confirmation")
            msg.setText(f"Please confirm deletion of :\n{names}")
            msg.setStandardButtons(QtWidgets.QMessageBox.Apply | QtWidgets.QMessageBox.Cancel)
            if msg.exec_() == QtWidgets.QMessageBox.Apply:
                self._delete(files)
        else:
            self._delete(files)

    def _findDefaultMusicFolder(self):
        xdgDirectories = os.path.expanduser("~/.config/user-dirs.dirs")
        if os.path.isfile(xdgDirectories):
            with open(xdgDirectories) as f:
                for line in f.read().splitlines():
                    line = line.split("=")
                    if line[0] == "XDG_MUSIC_DIR":
                        musicFolder = line[1].replace("$HOME", "~")
                        musicFolder = os.path.expanduser(musicFolder[1:-1])
                        self.preferences.db["general"]["music database"].append(musicFolder)
                        self.preferences.save()
                        break

    def _initStyle(self):
        iconTheme = self.preferences.get("general", "icon theme")
        iconPath = f"{LOCAL_DIR}icons/{iconTheme}/"
        self.icon = {}
        for icon in os.listdir(iconPath):
            iconName = os.path.splitext(icon)
            if iconName[1] == ".svg":
                self.icon[iconName[0]] = QtGui.QIcon(f"{iconPath}{icon}")
        self.ui.playButton.setIcon(self.icon["play"])
        self.ui.stopButton.setIcon(self.icon["stop"])
        self.ui.nextButton.setIcon(self.icon["next"])
        self.ui.previousButton.setIcon(self.icon["previous"])
        self.ui.preferencesButton.setIcon(self.icon["preferences"])
        self.ui.refreshButton.setIcon(self.icon["refresh"])
        self.ui.collapseButton.setIcon(self.icon["collapse"])
        self.ui.expandButton.setIcon(self.icon["expand"])
        if self.preferences.get("state", "shuffle"):
            self.ui.shuffleButton.setIcon(self.icon["shuffle_on"])
        else:
            self.ui.shuffleButton.setIcon(self.icon["shuffle_off"])

        # Init stylesheet
        stylesheet = f"QPushButton#libraryFilterClearButton {{ border-image: url({iconPath}filter_clear.svg); }}\n"
        stylesheet += f"QPushButton#libraryFilterClearButton:hover {{ border-image: url({iconPath}filter_hover.svg); }}"
        self.ui.libraryFilterClearButton.setStyleSheet(stylesheet)

    def _menuRefresh(self):
        self.menu.clear()
        if self.copied:
            self.menu.addAction(self.icon["paste"], "Paste selection", lambda: self.action("paste"))
        if self.tabWidget.current.selectedItems():
            self.menu.addAction(self.icon["cut"], "Cut selection", lambda: self.action("cut"))
            self.menu.addAction(self.icon["copy"], "Copy selection", lambda: self.action("copy"))
            if self.preferences.get("general", "file manager"):
                self.menu.addAction(self.icon["folder"], "Browse song folder", lambda: self.action("browse"))
            self.menu.addSeparator()
            self.menu.addAction(self.icon["remove"], "Delete from playlist", lambda: self.action("delete"))
            self.menu.addAction(self.icon["delete"], "Delete from disk", lambda: self.action("delete prompt"))

    def _resumePlayback(self):
        self.player.resumePlayback = self.preferences.get("general", "resume playback")
        playing = self.preferences.get("state", "media state") == QtMultimedia.QMediaPlayer.PlayingState
        if self.player.resumePlayback and playing:
            self.player.setCurrentMedia(self.preferences.get("state", "file"))

    def _titleFormat(self, title):
        replace = ("artist", "album", "track", "title")
        for tag in replace:
            title = title.replace("%" + tag + "%", self.player.tags[tag])
        return title

    def action(self, action):
        currentTab = self.tabWidget.current
        selection = currentTab.selectedItems()
        if action == "paste" and self.copied:
            for path in self.copied:
                item = currentTab.add(path)

            # Select pasted item
            currentTab.clearSelection()
            currentTab.setCurrentItem(item)

        elif selection:
            files = []
            for item in selection:
                files.append(item.text(5))

            if action == "copy":
                self.copied = files

            if action == "cut":
                self.copied = files
                self.action("delete")

            elif action == "delete":
                for item in selection:
                    index = currentTab.indexOfTopLevelItem(item)
                    currentTab.takeTopLevelItem(index)

                # Select next available item
                count = currentTab.topLevelItemCount()
                if index == count: index = count-1
                item = currentTab.topLevelItem(index)
                currentTab.clearSelection()
                currentTab.setCurrentItem(item)

            elif action == "delete prompt":
                self._deletePrompt(files)

            elif action == "browse":
                folder = os.path.abspath(os.path.join(currentTab.currentItem().text(5), os.pardir))
                cmd = f'{self.preferences.get("general", "file manager")} "{folder}"'
                self.process.startDetached(cmd)

    def currentMediaChanged(self, path):
        titlePopup = self._titleFormat(self.preferences.get("viewer", "notification format"))
        titleWindow = self._titleFormat(self.preferences.get("viewer", "title format"))
        titleTooltip = self._titleFormat(self.preferences.get("viewer", "tooltip format"))
        self.popup.display(titlePopup)
        self.setWindowTitle(titleWindow)
        self.trayIcon.setToolTip(titleTooltip)
        self._artLoad(path)
        if self.preferences.get("art", "media trigger"):
            self._artDisplay()

    def libraryDone(self):
        self.library.addWildCard()

    def librarySelect(self, path):
        self.player.lastPlayed = []
        self.tabWidget.current.sort()
        self.tabWidget.tabBar().setCurrentIndex(0)
        self._artLoad(path)
        if self.preferences.get("art", "library trigger"):
            self._artDisplay()

    def menuShow(self):
        if self.tabWidget.current.selectedItems() or self.copied:
            self.menu.popup(QtGui.QCursor.pos())

    def parseCommands(self, cmd):
        if "play-pause" in cmd:
            self.player.playPauseEvent()

        elif "stop" in cmd:
            self.player.stopEvent()

        elif "previous" in cmd:
            self.player.previousEvent()

        elif "next" in cmd:
            self.player.nextEvent()

        elif "quit" in cmd:
            self.close()

        elif "delete" in cmd:
            self.action("delete prompt")

        elif "shuffle" in cmd:
            if len(cmd["shuffle"]) > 0:
                if cmd["shuffle"][0] == "on":
                    self.player.shuffleEvent(enable=True)
                elif cmd["shuffle"][0] == "off":
                    self.player.shuffleEvent(enable=False)
            else:
                self.player.shuffleEvent()
            self.popup.display("Shuffle enabled" if self.player.shuffle else "Shuffle disabled")

        elif "file" in cmd:
            if len(cmd["file"]) == 0:
                return False

            self.tabWidget.tabs["Library viewer"]["playlist"].clear()
            for f in cmd["file"]:
                if os.path.isfile(f):
                    self.tabWidget.tabs["Library viewer"]["playlist"].add(f)
            self.tabWidget.tabBar().setCurrentIndex(0)
            self.tabWidget.tabs["Library viewer"]["playlist"].sort()
            self.player.activateSelection()

        elif "folder" in cmd:
            if len(cmd["folder"]) == 0:
                return False

            self.tabWidget.tabs["Library viewer"]["playlist"].clear()
            for folder in cmd["folder"]:
                if os.path.isdir(folder):
                    for root, subfolder, files in os.walk(folder):
                        for f in files:
                            self.tabWidget.tabs["Library viewer"]["playlist"].add(f"{root}/{f}")
            self.tabWidget.tabBar().setCurrentIndex(0)
            self.tabWidget.tabs["Library viewer"]["playlist"].sort()
            self.player.activateSelection()

        elif cmd:
            self.log.error(f"Unkown command: {cmd}")
            return False
        return True

    def parseKey(self, modifier, key):
        ctrl = (modifier == QtCore.Qt.ControlModifier)
        if ctrl and key == QtCore.Qt.Key_C:
            self.action("copy")
        elif ctrl and key == QtCore.Qt.Key_X:
            self.action("cut")
        elif ctrl and key == QtCore.Qt.Key_V:
            self.action("paste")
        elif key == QtCore.Qt.Key_Left:
            self.player.skip(-5)
        elif key == QtCore.Qt.Key_Right:
            self.player.skip(5)
        elif key == QtCore.Qt.Key_Backspace:
            self.action("delete")
        elif key == QtCore.Qt.Key_Delete:
            self.action("delete prompt")
        elif key == QtCore.Qt.Key_Space:
            self.player.playPauseEvent()

    def preferencesSave(self):
        self.preferencesForm.settingsSave()
        oldDb = self.preferences.get("general", "music database")
        newDb = self.preferencesForm.db["general"]["music database"]
        changedDb = not oldDb == newDb

        self.preferences.db = tools.copyDict(self.preferencesForm.db)
        self.preferences.save()

        for path in oldDb:
            if path not in newDb:
                self.parser.removeDatabase(path)
        if changedDb:
            QtCore.QMetaObject.invokeMethod(self.parser, "scanAll", QtCore.Qt.QueuedConnection)

        self._initStyle()
        self._artDisplay()
        self.tabWidget.initStyle()
        self.popup.updateView()
        if self.preferences.get("general", "tray icon"):
            self.trayIcon.show()
        else:
            self.trayIcon.hide()

    def preferencesShow(self):
        self.preferencesForm.show()
        self.preferencesForm.raise_()

    def setItemColor(self, color):
        current = self.tabWidget.current.currentItem()
        if current:
            self.tabWidget.current.currentItem().setColor(color)

    def setStatusMessage(self, left, right):
        if left:
            self.ui.statusBar.showMessage(left)
        if right:
            self.ui.statusRightLabel.setText(right)


def main(cmd=""):
    app = QtWidgets.QApplication([''])
    app.setQuitOnLastWindowClosed(False)
    gui = Main(app)
    gui.parseCommands(cmd)
    bus = tools.QDBusObject(parent=gui)
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



#!/usr/bin/python3
import os
import setuptools
import setuptools.command.build_py
here = os.path.abspath(os.path.dirname(__file__))


class CreateDesktopFile(setuptools.command.build_py.build_py):
  def run(self):
    with open(os.path.join(here + "/qoob.desktop"), 'w') as f:
        f.write("[Desktop Entry]\n")
        f.write("Name=qoob\n")
        f.write("GenericName=Music Player\n")
        f.write("Terminal=false\n")
        f.write("Type=Application\n")
        f.write("Categories=Utility;\n")
        f.write("Icon=qoob\n")
        f.write("Exec=qoob\n")
    setuptools.command.build_py.build_py.run(self)


# Workaround in case PyQt5 was installed without pip
install_requires=['mutagen']
try:
    # Compile ui files to python
    from PyQt5 import uic
    uic.compileUiDir('qoob/ui')
except:
    install_requires.append("pyqt5")

setuptools.setup(
    name='qoob',
    version='0.0.9',
    description='foobar-like music player for Linux',
    keywords='music folder player audio media linux',
    author='William Belanger',
    url='https://gitlab.com/william.belanger/qoob',

    # From https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    cmdclass={'build_py': CreateDesktopFile},
    data_files=[
        ('share/icons/hicolor/scalable/apps', ['qoob/icons/qoob.svg']),
        ('share/applications', ['qoob.desktop'])
    ],
    package_data={'': ['icons/*', 'icons/dark/*.svg', 'icons/light/*.svg']},
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'qoob=qoob:main',
        ],
    },
)



#!/usr/bin/python3
import os
from PyQt5 import QtGui, QtWidgets, QtCore

try:
    import qoob.backend.tools as tools
    import qoob.backend.parser as parser
except ImportError:
    import backend.tools as tools
    import backend.parser as parser

LOCAL_DIR = os.path.dirname(os.path.realpath(__file__)) + "/"
DB_DIR = os.path.expanduser("~/.config/qoob/")
DEFAULT = \
{
    "sort enabled": False,
    "sort order": QtCore.Qt.AscendingOrder,
    "sort column": 0,
    "column width": {0: 200, 1:200, 2:80, 4:80},
    "current": "",
    "files": []
}


class SortBox(QtWidgets.QCheckBox):
    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.setText("Enable sorting")
        self.clicked.connect(self._toggle)
        if parent.preferences.get("viewer", "sort by default"):
            self.setChecked(True)

    def _toggle(self):
        self.parent.tabWidget.current.setSortingEnabled(self.isChecked())


class Tabs(QtWidgets.QTabWidget):
    keyEvent = QtCore.pyqtSignal(object, int)
    changed = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.preferences = parent.preferences

        self.setDocumentMode(True)
        self.setTabsClosable(True)
        self.setMovable(True)

        self.tab1 = QtWidgets.QWidget()
        self.tab2 = QtWidgets.QWidget()
        self.addTab(self.tab1, "Library viewer")
        self.addTab(self.tab2, "")

        self.setTabEnabled(1, False)
        self.dummyButton = QtWidgets.QPushButton()
        self.dummyButton.setFixedSize(0, 0)
        self.newButton = QtWidgets.QPushButton()
        self.newButton.setFlat(True)
        self.newButton.setFocusPolicy(QtCore.Qt.NoFocus)
        self.newButton.setFixedSize(30, 20)
        self.newButton.clicked.connect(self._add)
        self.newButton.setToolTip("New playlist")
        self.tabBar().setTabButton(0, QtWidgets.QTabBar.RightSide, self.dummyButton)
        self.tabBar().setTabButton(1, QtWidgets.QTabBar.RightSide, self.newButton)

        self.tabCloseRequested.connect(self._close)
        self.currentChanged.connect(self._changed)
        self.installEventFilter(self)
        self.tabBar().installEventFilter(self)

        self.tabs = {}
        self.tabs["Library viewer"] = {"playlist": Tree(parent), "sort": SortBox(parent)}
        self.current = self.tabs["Library viewer"]["playlist"]
        self.initStyle()

        self.viewLayout = QtWidgets.QGridLayout()
        self.viewLayout.addWidget(self.tabs["Library viewer"]["playlist"])
        self.viewLayout.addWidget(self.tabs["Library viewer"]["sort"])
        self.currentWidget().setLayout(self.viewLayout)

        self.playlist = tools.Database("playlist")
        self._playlistsInit()
        self._playlistsLoadAttributes()
        self._playlistsLoadContent()

    def eventFilter(self, obj, event):
        if obj == self.tabBar() and event.type() == QtCore.QEvent.MouseButtonDblClick:
            self._rename()
        elif obj == self and event.type() == QtCore.QEvent.ShortcutOverride:
            self.keyEvent.emit(event.modifiers(), event.key())
        return QtCore.QObject.event(obj, event)

    def _add(self, name=None):
        layout = QtWidgets.QVBoxLayout()
        tab = QtWidgets.QWidget(self)
        count = self.tabBar().count()
        index = 1
        if not name:
            name = "Playlist " + str(index)
        while name in self.tabs:
            index += 1
            name = "Playlist " + str(index)

        self.tabs[name] = {}
        self.tabs[name] = {"playlist": Tree(self.parent), "sort": SortBox(self.parent)}
        self.tabs[name]["playlist"].setSortingEnabled(self.tabs[name]["sort"].isChecked())
        header = self.tabs["Library viewer"]["playlist"].header()
        for column in (0, 1, 2, 4):
            self.tabs[name]["playlist"].setColumnWidth(column, header.sectionSize(column))

        self.addTab(tab, name)
        self.setCurrentWidget(tab)
        self.tabBar().moveTab(count - 1, count)
        self.currentWidget().setLayout(layout)
        layout.addWidget(self.tabs[name]["playlist"])
        layout.addWidget(self.tabs[name]["sort"])

    def _changed(self, index):
        tabName = self.tabText(index)
        tabName = tabName.lstrip("&")  # KDE bug fix (auto append of &)
        if tabName:
            self.current = self.tabs[tabName]["playlist"]
            self.changed.emit()

    def _close(self, tab):
        tabName = self.tabText(tab)
        tabName = tabName.lstrip("&")  # KDE bug fix (auto append of &)
        del self.tabs[tabName]
        if tabName in self.playlist.db:
            del self.playlist.db[tabName]
            self.playlist.save()
        self.removeTab(tab)

        # Prevent landing on the '+' tab
        count = self.tabBar().count() - 1
        if count == self.tabBar().currentIndex():
            self.tabBar().setCurrentIndex(count - 1)

    def _playlistDumpAttributes(self, tabName):
        self.playlist.db[tabName]["sort enabled"] = self.tabs[tabName]["sort"].isChecked()
        self.playlist.db[tabName]["sort order"] = self.tabs[tabName]["playlist"].header().sortIndicatorOrder()
        self.playlist.db[tabName]["sort column"] = self.tabs[tabName]["playlist"].sortColumn()
        for column in (0, 1, 2, 4):
            width = self.tabs[tabName]["playlist"].header().sectionSize(column)
            self.playlist.db[tabName]["column width"][column] = width

    def _playlistDumpFiles(self, tabName):
        if self.tabs[tabName]["playlist"].currentItem():
            self.playlist.db[tabName]["current"] = self.tabs[tabName]["playlist"].currentItem().text(5)

        for item in range(self.tabs[tabName]["playlist"].topLevelItemCount()):
            item = self.tabs[tabName]["playlist"].topLevelItem(item)
            self.playlist.db[tabName]["files"].append(item.text(5))

    def _playlistsInit(self):
        for tabName in self.playlist.db:
            if not tabName == "Library viewer":
                self._add(tabName)

    def _playlistsLoadAttributes(self):
        for tabName in self.playlist.db:
            sortEnabled = self.playlist.db[tabName].get("sort enabled", DEFAULT["sort enabled"])
            sortOrder = self.playlist.db[tabName].get("sort order", DEFAULT["sort order"])
            sortColumn = self.playlist.db[tabName].get("sort column", DEFAULT["sort column"])
            columnWidth = self.playlist.db[tabName].get("column width", DEFAULT["column width"])
            self.tabs[tabName]["playlist"].sortByColumn(sortColumn, sortOrder)
            self.tabs[tabName]["sort"].setChecked(sortEnabled)
            self.tabs[tabName]["playlist"].setSortingEnabled(sortEnabled)
            for column in columnWidth:
                self.tabs[tabName]["playlist"].setColumnWidth(int(column), columnWidth[column])

    def _playlistsLoadContent(self):
        for tabName in self.playlist.db:
            for path in self.playlist.db[tabName]["files"]:
                item = self.tabs[tabName]["playlist"].add(path, append=True)
                if "current" in self.playlist.db[tabName] and path == self.playlist.db[tabName]["current"]:
                    self.tabs[tabName]["playlist"].setCurrentItem(item)
        self.tabBar().setCurrentIndex(self.preferences.get("state", "tab"))

    def _rename(self):
        tabIndex = self.tabBar().currentIndex()
        oldName = self.tabText(tabIndex)
        oldName = oldName.lstrip("&")  # KDE bug fix (auto append of &)
        if not oldName == "Library viewer":
            newName = self._renamePrompt(oldName)
            if newName and newName not in self.tabs:
                self.tabs[newName] = self.tabs.pop(oldName)
                if oldName in self.playlist.db:
                    self.playlist.db[newName] = self.playlist.db.pop(oldName)
                    self.playlist.save()
                self.setTabText(tabIndex, newName)

    def _renamePrompt(self, oldName):
        msg = QtWidgets.QInputDialog()
        msg.setInputMode(QtWidgets.QInputDialog.TextInput)
        msg.setWindowFlags(msg.windowFlags() | QtCore.Qt.WindowStaysOnTopHint)
        msg.setWindowTitle(f"Rename '{oldName}'")
        msg.setLabelText("Enter the new name:")
        msg.setTextValue(oldName)
        msg.setFixedSize(250, 100)
        accept = msg.exec_()
        newName = msg.textValue()
        newName = newName.replace("&", "")  # KDE bug fix
        if accept and newName:
            return newName

    def initStyle(self):
        iconTheme = self.preferences.get("general", "icon theme")
        iconPath = f"{LOCAL_DIR}../icons/{iconTheme}/"
        stylesheet = f"QTabBar::close-button {{ image: url({iconPath}tab_close.svg); }}\n"
        stylesheet += f"QTabBar::close-button:hover {{ image: url({iconPath}tab_hover.svg); }}\n"
        self.setStyleSheet(stylesheet)
        self.newButton.setIcon(self.parent.icon["tab_add"])

    def playlistSave(self):
        for index in range(self.count() - 1):
            tabName = self.tabText(index)
            tabName = tabName.lstrip("&")  # KDE bug fix (auto append of &)
            self.playlist.db[tabName] = tools.copyDict(DEFAULT)
            self._playlistDumpAttributes(tabName)
            self._playlistDumpFiles(tabName)
        self.playlist.save()


class Tree(QtWidgets.QTreeWidget):
    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.preferences = parent.preferences
        self.metadata = parent.parser.metadata

        self.setHeaderLabels(["Artist", "Album", "Track", "Title", "Duration"])
        self.header().setSectionResizeMode(3, QtWidgets.QHeaderView.Stretch)
        self.header().setStretchLastSection(False)
        self.setAlternatingRowColors(True)
        self.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(parent.menuShow)
        self.itemActivated.connect(parent.player.activateSelection)

    def keyPressEvent(self, event):
        # Block default behavior (lateral tree scrolling)
        if not event.key() == QtCore.Qt.Key_Left and not event.key() == QtCore.Qt.Key_Right:
            QtWidgets.QTreeWidget.keyPressEvent(self, event)

    def add(self, path, append=False):
        basename = os.path.basename(path)
        extension = os.path.splitext(basename)[1].lower()
        if extension in parser.ALLOWED_AUDIO_TYPES:
            tags = self.parent.parser.header(path)
            item = TreeItem()
            item.setText(0, tags["artist"])
            item.setText(1, tags["album"])
            item.setText(2, tags["track"])
            item.setText(3, tags["title"])
            item.setText(4, tags["duration"])
            item.setText(5, path)

            if append:
                self.addTopLevelItem(item)
            else:
                index = self.currentIndex().row()
                if index == -1: index = 0
                self.insertTopLevelItem(index, item)
            return item

    def sort(self):
        headers = {"Artist": 0, "Album": 1, "Track": 2, "Title": 3, "Duration": 4}
        sortingRoutine = []

        for action in self.preferences.get("viewer", "sorting routine"):
            if action == "Disable sorting":
                self.parent.tabWidget.tabs["Library viewer"]["sort"].setChecked(False)
                self.setSortingEnabled(False)
            else:
                self.parent.tabWidget.tabs["Library viewer"]["sort"].setChecked(True)
                action = action.split()
                header = headers[action[0]]
                if action[1] == "(Ascending)":
                    direction = QtCore.Qt.AscendingOrder
                elif action[1] == "(Descending)":
                    direction = QtCore.Qt.DescendingOrder
                self.setSortingEnabled(True)
                self.sortByColumn(header, direction)


class TreeItem(QtWidgets.QTreeWidgetItem):
    def __init__(self):
        super().__init__()
        self.setFlags(self.flags() &~ QtCore.Qt.ItemIsDropEnabled)
        self.lastColor = "none"

    def __lt__(self, other):
        column = self.treeWidget().sortColumn()
        key1 = self.text(column)
        key2 = other.text(column)
        try:
            return int(key1) < int(key2)
        except ValueError:
            return key1.lower() < key2.lower()

    def setColor(self, color):
        priority = {"none": 0, "green": 0, "yellow": 1, "red": 2}
        if priority[color] >= priority[self.lastColor]:
            self.lastColor = color
            try:
                for column in range(5):
                    if color == "none":
                        self.setData(column, QtCore.Qt.BackgroundRole, None)
                        self.setData(column, QtCore.Qt.ForegroundRole, None)
                    elif color == "green":
                        self.setForeground(column, QtGui.QColor("#004000"))
                        self.setBackground(column, QtGui.QColor("#c6efce"))
                    elif color == "yellow":
                        self.setForeground(column, QtGui.QColor("#553400"))
                        self.setBackground(column, QtGui.QColor("#ffeb9c"))
                    elif color == "red":
                        self.setForeground(column, QtGui.QColor("#9c0006"))
                        self.setBackground(column, QtGui.QColor("#ffc7ce"))
            except RuntimeError:
                pass





#!/usr/bin/python3
import os
from PyQt5 import QtGui, QtWidgets, QtCore, QtMultimedia


class PopupWidget(QtWidgets.QWidget):
    def __init__(self, parent):
        super().__init__()
        self.preferences = parent.preferences
        self.textLabel = QtWidgets.QLabel()
        self.textLabel.setWordWrap(True)
        self.textLabel.setAlignment(QtCore.Qt.AlignCenter)
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.textLabel)
        layout.setAlignment(self.textLabel, QtCore.Qt.AlignCenter)
        self.setLayout(layout)

        self.hideTimer = QtCore.QTimer(singleShot=True)
        self.hideTimer.timeout.connect(self.hide)

        # Hide window from system taskbar
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint|QtCore.Qt.FramelessWindowHint)
        self.setAttribute(QtCore.Qt.WA_ShowWithoutActivating)
        if os.environ.get("DESKTOP_SESSION") == "openbox":
            self.setAttribute(QtCore.Qt.WA_X11NetWmWindowTypeToolBar)
        else:
            self.setAttribute(QtCore.Qt.WA_X11NetWmWindowTypeUtility)
        self.setMouseTracking(True)
        self.updateView()

    def mouseMoveEvent(self, event):
        self.hide()

    def mousePressEvent(self, event):
        self.hide()

    def _setFontSize(self, message):
        font = self.textLabel.font()
        font.setPointSize(self.preferences.get("popup", "font size"))
        outside = self.contentsRect()
        inside = outside.marginsRemoved(QtCore.QMargins(1,5,1,5))

        while font.pointSize() > 5:
            size = QtGui.QFontMetrics(font).boundingRect(outside, QtCore.Qt.TextWordWrap, message)
            if inside.width() >= size.width() and inside.height() >= size.height():
                break
            else:
                font.setPointSize(font.pointSize()-1)
        self.textLabel.setFont(font)

    def _setGeometry(self):
        width = self.preferences.get("popup", "width")
        height = self.preferences.get("popup", "height")
        self.setFixedSize(width, height)

    def _setPosition(self):
        available = QtWidgets.QDesktopWidget().availableGeometry()
        position = self.frameGeometry()
        move = self.preferences.get("popup", "position")

        if move == "top left":
            position.moveTopLeft(available.topLeft())
        elif move == "top center":
            position.moveCenter(available.center())
            position.moveTop(available.top())
        elif move == "top right":
            position.moveTopRight(available.topRight())
        elif move == "bottom left":
            position.moveBottomLeft(available.bottomLeft())
        elif move == "bottom center":
            position.moveCenter(available.center())
            position.moveBottom(available.bottom())
        elif move == "bottom right":
            position.moveBottomRight(available.bottomRight())
        elif move == "middle left":
            position.moveCenter(available.center())
            position.moveLeft(available.left())
        elif move == "middle center":
            position.moveCenter(available.center())
        elif move == "middle right":
            position.moveCenter(available.center())
            position.moveRight(available.right())

        x = position.x() + self.preferences.get("popup", "horizontal offset")
        y = position.y() + self.preferences.get("popup", "vertical offset")
        self.move(x, y)

    def _setStyle(self):
        fg = QtGui.QColor(self.preferences.get("popup", "foreground"))
        bg = QtGui.QColor(self.preferences.get("popup", "background"))
        palette = QtGui.QPalette()
        palette.setColor(QtGui.QPalette.WindowText, fg);
        palette.setColor(QtGui.QPalette.Window, bg);
        self.setPalette(palette)
        self.textLabel.setPalette(palette)
        self.setWindowOpacity(self.preferences.get("popup", "opacity"))

        font = QtGui.QFont()
        font.setFamily(self.preferences.get("popup", "font family"))
        self.textLabel.setFont(font)

    def display(self, message):
        if self.preferences.get("popup", "enable"):
            width = int(self.preferences.get("popup", "width") * 0.6)
            height = int(self.preferences.get("popup", "height") * 0.6)
            self.textLabel.setText(message)
            self._setFontSize(message)
            self.hideTimer.start(1000 * self.preferences.get("popup", "duration"))
            self.show()

    def updateView(self):
        self._setStyle()
        self._setGeometry()
        self._setPosition()


class TrayIcon(QtWidgets.QSystemTrayIcon):
    hideWindow = QtCore.pyqtSignal()
    showWindow = QtCore.pyqtSignal()
    close = QtCore.pyqtSignal()
    play = QtCore.pyqtSignal()
    stop = QtCore.pyqtSignal()
    next = QtCore.pyqtSignal()
    previous = QtCore.pyqtSignal()
    shuffle = QtCore.pyqtSignal()
    showPreferencesForm = QtCore.pyqtSignal()
    popup = QtCore.pyqtSignal(str)

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.icon = parent.icon
        self.preferences = parent.preferences
        self.setIcon(QtGui.QIcon(self.icon["qoob"].pixmap(64, 64)))  # SVG to pixmap conversion (KDE compatibility)
        self.activated.connect(self._clickEvent)
        self._menuSetup()
        if self.preferences.get("general", "tray icon"):
            self.show()

    def _clickEvent(self, event):
        if event == QtWidgets.QSystemTrayIcon.Trigger:
            if self.preferences.get("general", "tray minimize"):
                self.hideWindow.emit() if self.parent.isVisible() else self.showWindow.emit()
        elif event == QtWidgets.QSystemTrayIcon.MiddleClick:
            self.play.emit()
            if self.parent.player.state() == QtMultimedia.QMediaPlayer.PausedState:
                self.popup.emit("Paused")

    def _menuSetup(self):
        menu = QtWidgets.QMenu()
        menu.addAction(self.icon["play"], "Play/Pause", self.play.emit)
        menu.addAction(self.icon["next"], "Next", self.next.emit)
        menu.addAction(self.icon["previous"], "Previous", self.previous.emit)
        menu.addAction(self.icon["shuffle_on"], "Shuffle", self._shuffleTrayEvent)
        menu.addAction(self.icon["stop"], "Stop", self.stop.emit)
        menu.addSeparator()
        menu.addAction(self.icon["preferences"], "Preferences", self.showPreferencesForm.emit)
        menu.addAction(self.icon["quit"], "Quit", self.close.emit)
        self.setContextMenu(menu)

    def _shuffleTrayEvent(self):
        self.shuffle.emit()
        self.popup.emit("Shuffle enabled" if self.parent.player.shuffle else "Shuffle disabled")
